/********************************************************************
 ********************************************************************
 * FileName:     BLE_LoopBack.c
 * Dependencies:
 * Processor:	 PIC24
 * Complier:     Microchip xc16 ver 1.25
 * Company:      Electronic Design Solutions, LLC.
 *
 *********************************************************************
 * File Description:
 *
 *  This file handles the Bluetooth Low Energy loopback demo
 *
 * Change History:
 *  Rev   Date         Author    Description
 *  1.0   10/23/2015    DRW      Initial revision
 ********************************************************************/
#include <stdlib.h>

#include <string.h>
#include <stdio.h>
#include "resources/font_resources_epd_demo.h"
#include "GenericTypeDefs.h"
#include "rtcc.h"
#include "Utilities.h"
#include "system.h"
#include "system_config.h"
#include "../../../../Microchip/mla/v2015_05_15/framework/miwi/miwi_api.h"
#include "Bluetooth.h"
#include "driver/uart/drv_uart1.h"

void validate_stringCharaters(char *, BYTE);

#define BLE_INITIALIZE     0X01
#define BLE_CONNECT        0X02
#define BLE_PAIRED         0x03
#define BLE_LOOPBACK       0x04
#define BLE_DELAY_DISPLAY  0x05
#define BLE_UPDATE_SCREEN  0x06

extern GFX_GOL_OBJ_SCHEME pScheme_button;
extern GFX_GOL_OBJ_SCHEME VB10_Scheme;
extern  uint8_t responseBuffer[25];
extern DWORD displayChangeTime;
extern APP_SCREEN_UPDATE_STATES update_display;
BYTE BLE_connection_state;
char ble_string[25];
uint32_t delay_1second;

/****************************************************************************
  Function:
    void ShowScreenBLE_LoopBack( void )

  Description:
    This function erases the currently displayed screen and replaces it with
    a screen consisting of:
 * the Microchip icon
 * a one line title
 * 3 line of instructions

    The information for the screen contents is obtained from the global
    pageInformation structure.

  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Remarks:
    The global list of graphics objects is destroyed and replaced.  Therefore,
    this function cannot be called during the message processing of
    GOLMsgCallback().  It must be called during GOLDrawCallback() processing.
 ***************************************************************************/
void ShowScreenBLE_LoopBack(void) {
    strcpy(pageInformation.title, "LOOPBACK");
    InitializeScreen(); // set up logo and title text and destroys link list
    // of objects
    strcpy(pageInformation.instructions2, "Please Connect ");
    strcpy(pageInformation.instructions1, " ");
    strcpy(pageInformation.instructions, "");
    strcpy(pageInformation.instructions3, " ");
    strcpy(pageInformation.buttonTextBottomLeft, " OK");

    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS_2,
            0, 21, 127, 21 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10) + 4,
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions2,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);

    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS_1,
            0, 34, 127, 34 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10) + 4,
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions1,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);


    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS,
            0, 47, 127, 47 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10) + 4,
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);


    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS_3,
            0, 61, 127, 61 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10) + 4,
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions3,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);


    GFX_GOL_ButtonCreate(ID_BUTTON_GENERIC_HOME, // ID
            49, 81, 74, 95, 2,                // dimension and radius
            GFX_GOL_BUTTON_DRAW_STATE ,       // will be displayed after creation
            NULL,                             // bitmap
            NULL,                             // no release image used
            (GFX_XCHAR*) pageInformation.buttonTextBottomLeft, // text
            GFX_ALIGN_HCENTER | GFX_ALIGN_VCENTER,
            &pScheme_button); //

}

/****************************************************************************
  Function:
    bool ProcessMessage_BLE_LoopBack( WORD translatedMsg, GFX_GOL_OBJ_HEADER* pObj,
                        GFX_GOL_MESSAGE* pMsg )

  Description:
    This function processes the messages for the BLE_LoopBack test selection screen.  The
    global screenState is updated to proceed to the user's selection.  

  Precondition:
    Call ShowScreenBLE_LoopBack(); prior to using this function to display the
    correct screen.

  Parameters:
    WORD translatedMsg  - The translated control-level message
    GFX_GOL_OBJ_HEADER* pObj    - Object to which the message applies
    GFX_GOL_MESSAGE* pMsg       - The original system message information

  Return Values:
    0   - Do not call the default message handler.
    1   - Call the default message handler.

  Remarks:
 * press the center button to end the demo
    
 ***************************************************************************/
bool ProcessMessage_BLE_LoopBack(WORD translatedMsg, GFX_GOL_OBJ_HEADER* pObj, GFX_GOL_MESSAGE* pMsg) {
    WORD controlID;
    controlID = GFX_GOL_ObjectIDGet(pObj);
    switch (controlID) {
        case ID_BUTTON_GENERIC_HOME:
            if (translatedMsg == GFX_GOL_BUTTON_ACTION_PRESSED) {
                screenState = SCREEN_DISPLAY_MAIN;
                Bluetooth_disconnect();
                return false; // Hidden button

            } else {
                return false;
            }
            break;
        default:
            return false;
    }

    return true; // Call the default handler to show the button state.
}


/****************************************************************************
  Function:
    void BLE_state_initilize( void )

  Description:
    set the state machine variable
  Precondition:
    UART should be working

  Parameters:
    none

  Return Values:
    none

  Remarks:
   none
 ***************************************************************************/
void BLE_state_initilize(){
    BLE_connection_state = BLE_INITIALIZE;
}


/****************************************************************************
  Function:
    bool Bluetooth_LoopBack( void )

  Description:
    This function configures the bluetooth module to operate in MLDP mode.
 * It handles the connect and disconnect messages.  The incoming messages
 * are sent to the display as well as echoed back to the smart device.

  Precondition:
    UART should be working

  Parameters:
   

  Return Values:
    none

  Remarks:
 The MLDP app hangs if the does not connect properly.  Error "MLDP mode not supported"
 * close and restart the app or reboot the smart device.  It is possible to get 
 * trash characters over the air.  This routine filters out trash characters
 * and replaces them with "?"
 ***************************************************************************/
void Bluetooth_LoopBack(void) {
    BYTE writebufferLen;
    int i;

    DRV_UART1_TasksRX();    // fill circular buffer _rxByteQ from the  4 deep fifo 
    DRV_UART1_TasksTX();    // send out from circular buffer _txByteQ to the 4 deep TX fifo
    DRV_UART1_TasksError(); // clear uart rx over run
    switch (BLE_connection_state) {
        case BLE_INITIALIZE:
            if (configure_Bluetooth_MLDP() == true)
                BLE_connection_state = BLE_CONNECT;
            break;
        case BLE_CONNECT:
            if (getResponse()) {
                if (strncmp((char *) responseBuffer, "Connected", 9) == 0) {
                    BLE_connection_state = BLE_PAIRED;
                    strcpy(pageInformation.instructions2, "Connected ");
                    pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_2);
                    GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions2);
                    GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                    update_display = GLOBAL_UPDATE;
                } else
                    DRV_UART1_Flush_RX_buffer();
            }
            break;
        case BLE_PAIRED:
            send_uart_command(responseBuffer);  // send out "Connected"
            BLE_connection_state = BLE_LOOPBACK;
            break;
        case BLE_LOOPBACK:
            if (getResponse()) {

                if (strncmp((char *) responseBuffer, "Connection End", 10) == 0) {
                    BLE_connection_state = BLE_CONNECT;
                    DRV_UART1_Flush_RX_buffer();
                    strcpy(pageInformation.instructions2, "Please Connect ");
                    pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_2);
                    GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions2);
                    GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                    strcpy(pageInformation.instructions1, " ");
                    pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_1);
                    GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions1);
                    GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                    update_display = GLOBAL_UPDATE;
                } else if (strncmp((char *) responseBuffer, "ERR", 3) == 0) {
                    BLE_connection_state = BLE_CONNECT;
                    DRV_UART1_Flush_RX_buffer();
                    strcpy(pageInformation.instructions2, "Please Connect ");
                    update_display = GLOBAL_UPDATE;
                    pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_2);
                    GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions2);
                    GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                } else {
                    writebufferLen = strlen((char *) responseBuffer) - 1;  // remove /n character
                    for (i = 0; i < 25; i++)ble_string[i] = 0;
                    validate_stringCharaters((char *) responseBuffer, writebufferLen);
                    strncpy(ble_string,  (char *) responseBuffer, writebufferLen);
                    DRV_UART1_Flush_RX_buffer();// problems with circular buffer at the end of the buffer
                    send_uart_command(responseBuffer);  // echo back 
                    BLE_connection_state = BLE_DELAY_DISPLAY;
                    delay_1second = tick + 10;
                }
            }
            break;
        case BLE_DELAY_DISPLAY:
            if (delay_1second < tick)BLE_connection_state = BLE_UPDATE_SCREEN;
            break;
        case BLE_UPDATE_SCREEN:
            sprintf((char *) &(pageInformation.instructions1), (char*) "%s", ble_string);
            pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_1);
            GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions1);
            GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
            update_display = PARTIAL_UPDATE;
            displayChangeTime = tick;
            BLE_connection_state = BLE_LOOPBACK;
            break;
        default:
            BLE_connection_state = BLE_INITIALIZE;

    }

}

// keep the graphics library from choking
// do not try to print values that are not defined
void validate_stringCharaters(char *buffer, BYTE length) {
    BYTE eachCharacter;
    for (eachCharacter = 0;  eachCharacter < length  ; eachCharacter++) {
        if (*(buffer + eachCharacter) < 32  || *(buffer + eachCharacter) > 122)
            *(buffer + eachCharacter) = '?';
    }
}