/********************************************************************
 ********************************************************************
 * FileName:     Battery_utility.c
 * Dependencies:
 * Processor:	 PIC24
 * Complier:     Microchip xc16 ver 1.25
 * Company:      Electronic Design Solutions, LLC.
 *
 *********************************************************************
 * File Description:
 *
 *  This file handles the battery utilities
 *
 * Change History:
 *  Rev   Date         Author    Description
 *  1.0   6/18/2014    DRW       Initial revision
 ********************************************************************/
#include <stdlib.h>

#include <string.h>
#include <stdio.h>
#include "resources/font_resources_epd_demo.h"
#include "GenericTypeDefs.h"
#include "rtcc.h"
#include "Battery_utility.h"
#include "Utilities.h"
#include <i2c.h>
#include "system.h"
#include "system_config.h"
#include "../../../../Microchip/mla/v2015_05_15/framework/miwi/miwi_api.h"
#include "Pervasive_Displays_small_EPD/EPD_hardware_driver.h"
#include "revision_history.h"


CHARGING_STATES chargerState = EXTERNAL_PWR_APPLIED;

BYTE BatteryLevel;
BYTE old_status = 99;
extern GFX_GOL_OBJ_SCHEME pScheme_button;
extern GFX_GOL_OBJ_SCHEME VB10_Scheme;

extern BYTE UID48[6];
extern APP_SCREEN_UPDATE_STATES update_display;


XCHAR reset_codes[9][5] = {
    { 'T', 'R', 'A', 'P', '\0', },
    { 'I', 'O', 'P', 'U', '\0', },
    { 'C', 'M', ' ', ' ', '\0', },
    { 'E', 'X', 'T', 'R', '\0', },
    { 'S', 'W', 'R', ' ', '\0', },
    { 'W', 'D', 'T', 'O', '\0', },
    { 'B', 'O', 'R', ' ', '\0', },
    { 'P', 'O', 'R', ' ', '\0', },
    { 'I', 'C', 'D', ' ', '\0', }
};


XCHAR INT_codes[6][5] = {
    { 'M', 'A', 'T', 'H', '\0', },
    { 'A', 'D', 'D', 'R', '\0', },
    { 'S', 'T', 'K', 'E', '\0', },
    { 'O', 'S', 'C', ' ', '\0', },
    { 'm', 'i', 's', 'c', '\0', },
    { 'n', 'o', 'n', 'e', '\0', }
};

/*********************************************************************
 * Function:        BYTE resetChargeAccumulator(void)
 *
 * PreCondition:    I2c  must be configured and running
 *
 * Input:           None
 *
 * Output:          .
 *                 .
 *
 * Side Effects:
 *
 * Overview:        This function resets the gas gauge to ec77(60535)
 *                  after the charge is complete
 *
 *
 * Note:
 ********************************************************************/
void resetChargeAccumulator(void) {
    char i2cData[5];
    char DataSz;
    int Index = 0;
    BYTE I2cAddress;

    I2cAddress = 0b01100100;
    i2cData[0] = (I2cAddress << 1) | 0; //Device Address & WR
    i2cData[1] = GAS_GAUGE_ACCUMULATE_CHARGE_MSB;
    i2cData[2] = 0xEC; //data to write (one byte)  msb
    i2cData[3] = 0x77; //                          lsb
    DataSz = 4;

    StartI2C3(); //Send the Start Bit
    IdleI2C3();  //Wait to complete

    while (DataSz) {
        MasterWriteI2C3(i2cData[Index++]);
        IdleI2C3(); //Wait to complete
        DataSz--;
        //ACKSTAT is 0 when slave acknowledge,
        //if 1 then slave has not acknowledge the data.
        if (I2C3STATbits.ACKSTAT)
            break;
    }

    StopI2C3(); //Send the Stop condition
    IdleI2C3(); //Wait to complete
}

/*********************************************************************
 * Function:        void setControlRegister(void)
 *
 * PreCondition:    I2c 1 must be configured and running
 *
 * Input:           None
 *
 * Output:          
 *                
 *
 * Side Effects:
 *
 * Overview:        This function sets up the gas gauge to alert when the
 *                  battery voltage drops below 2.9 volts
 *
 *
 * Note:           B[7:6] VBAT Alert                  default  [00]
 *                        [11] Threshold Value = 3.0V.
 *                        [10] Threshold Value = 2.9V.
 *                        [01] Threshold Value = 2.8V.
 *                        [00] VBAT Alert Off.
 *
 *                 B[5:3] Prescaler M Sets coulomb counter prescaling  default [111]
 *                       factor M between 1 and 128.
 *                       M = 2(4 • B[5] + 2 • B[4] + B[3]).
 *
 *                 B[2:1] AL/CC  Configures the AL/CC pin.    default [10]
 *                        [10] Alert Mode. Alert functionality enabled.
 *                             Pin becomes logic output.
 *                        [01] Charge Complete Mode.
 *                             Pin becomes logic input and accepts
 *                              “charge complete” signal (e.g., from
 *                             a charger) to set accumulated charge
 *                             Register to FFFFh.
 *                        [00] AL/CC pin disabled.
 *                        [11] Not allowed.
 *
 *                  B[0] Shutdown Shut down analog section to reduce
 *                       ISUPPLY.
 ********************************************************************/
bool setControlRegister(void) {
    char i2cData[5];
    char DataSz;
    int Index = 0;
    BYTE I2cAddress;
    bool returnValue = true;
    MI2C1_Clear_Intr_Status_Bit;
    I2cAddress = BATTERY_GAS_GAUGE_ADDRESS;
    i2cData[0] = (I2cAddress << 1) | 0; //Device Address & WR
    i2cData[1] = GAS_GAUGE_CONTROL;
    i2cData[2] = VBAT_ALERT3_0 | PRESCALER_M | CONFIGURE_PIN_ALERT | POWER_DOWN_OPERATE;
    DataSz = 3;

    IdleI2C3();                         // Wait to complete
    StartI2C3();                        // Send the Start Bit
    while (I2C3CONbits.SEN );           // Wait till Start sequence is completed
    MI2C3_Clear_Intr_Status_Bit;        // Clear interrupt flag
    while (DataSz) {
        MasterWriteI2C3(i2cData[Index++]);
        IdleI2C3();                     // Wait to complete
        DataSz--;
        //ACKSTAT is 0 when slave acknowledge,
        //if 1 then slave has not acknowledge the data.
        if (I2C3STATbits.ACKSTAT) {
            break;
            StopI2C3();                 // Send the Stop condition
            IdleI2C3();                 // Wait to complete
            returnValue = false;
        }
    }
    StopI2C3();                         //Send the Stop condition
    IdleI2C3();                         //Wait to complete
    return returnValue;
}

/*********************************************************************
 * Function:        void gauge_powerDown(void)
 *
 * PreCondition:    I2c must be configured and running
 *
 * Input:           None
 *
 * Output:          
 *                 
 *
 * Side Effects:
 *
 * Overview:        This function  sets the power down b(0) bit to 1
 *                  
 *
 * Note:          this will save 40 micro amps
 ********************************************************************/
void gauge_power(BYTE power) {
    char i2cData[5];
    char DataSz;
    int Index = 0;
    BYTE I2cAddress;

    I2cAddress = BATTERY_GAS_GAUGE_ADDRESS;
    i2cData[0] = (I2cAddress << 1) | 0; // Device Address & WR
    i2cData[1] = GAS_GAUGE_CONTROL;
    i2cData[2] = VBAT_ALERT2_9 | PRESCALER_M | CONFIGURE_PIN_ALERT | power;
    DataSz = 3;

    StartI2C3();                        // Send the Start Bit
    IdleI2C3();                         // Wait to complete
    while (DataSz) {
        MasterWriteI2C3(i2cData[Index++]);
        IdleI2C3();                     // Wait to complete
        DataSz--;
        //ACKSTAT is 0 when slave acknowledge,
        //if 1 then slave has not acknowledge the data.
        if (I2C3STATbits.ACKSTAT)
            break;
    }
    StopI2C3();                         // Send the Stop condition
    IdleI2C3();                         // Wait to complete
}

/*********************************************************************
 * Function:        BYTE configure_gasGauge_threshold(void)
 *
 * PreCondition:    I2c 1 must be configured and running
 *
 * Input:           None
 *
 * Output:          
 *                 
 *
 * Side Effects:
 *
 * Overview: This function sets up four threshold registers,
 *                     high msb  0x04
 *                     high lsb  0x05
 *                     low  msb  0x06
 *                     low  lsb  0x07
 *
 * Note:    prescale M calculation
 *          Rsense = 120 mOhms
 *          qLSB = .085mAh*(50mOhm/Rsense)*(M/128)
 *
 *          required qLSB = Qbatt/FFFF
 *                        = 1000mAH/65535
 *
 *
 *          M = (1000mAh/65535)*128*(120/(50*.085))
 *
 *                1000mAh * 128 *120      15,360,000
 *            = ---------------------  = ------------
 *               65535 *50 *.085mAh        278524
 *
 *            = 55
 *          choose M to be 64
 *
 *          Then qLSB = .018 mAh/count
 *
 *          number of counts to deplete 1000 mAh battery
 *          lower threshold = 60535-(1000/.018) = 4064
 *                set threshold to 6000
 ********************************************************************/
void configure_gasGauge_threshold(void) {
    char i2cData[10];
    char DataSz;
    int Index = 0;
    BYTE I2cAddress;


    I2cAddress = 0b01100100;
    i2cData[0] = (I2cAddress << 1) | 0; //Device Address & WR
    i2cData[1] = GAS_GAUGE_CHARGE_THRESHOLD_HIGH_MSB;
    i2cData[2] = 0xFF; // set high threshold msb
    i2cData[3] = 0xFE; // set high threshold lsb
    i2cData[4] = 0x17; // set low threshold msb (6000)
    i2cData[5] = 0x70; // set low threshold lsb

    DataSz = 6;

    StartI2C3();       // Send the Start Bit
    IdleI2C3();        // Wait to complete
    while (DataSz) {
        MasterWriteI2C3(i2cData[Index++]);
        IdleI2C3();    // Wait to complete
        DataSz--;
        //ACKSTAT is 0 when slave acknowledge,
        //if 1 then slave has not acknowledge the data.
        if (I2C3STATbits.ACKSTAT)
            break;
    }
    StopI2C3();        //Send the Stop condition
    IdleI2C3();        //Wait to complete
}

/*********************************************************************
 * Function:        unsigned int read_gasGauge(void)
 *
 * PreCondition:    I2c must be configured and running
 *
 * Input:           None
 *
 * Output:         returns the current battery charge value
 *
 *
 * Side Effects:
 *
 * Overview:        queries charge status register
 *
 *
 *
 * Note:
 ********************************************************************/
unsigned int read_gasGauge(void) {
    WORD_VAL accumulateCharge;

    gauge_power(OFF);  // briefly turn off to stop counters
    accumulateCharge.Val = read_C_D_register();
    gauge_power(ON);
    return accumulateCharge.Val;
}

unsigned int read_C_D_register(void) {
    char i2cData[5];
    char DataSz;
    int Index = 0;
    BYTE msb_columbCNT;
    BYTE lsb_columbCNT;
    WORD_VAL accumulateCharge;
    BYTE I2cAddress;

    I2cAddress = 0b01100100;
    i2cData[0] = (I2cAddress << 1) | 0; //Device Address &WR
    i2cData[1] = GAS_GAUGE_ACCUMULATE_CHARGE_MSB;
    DataSz = 2;

    StartI2C3();    // Send the Start Bit
    IdleI2C3();     // Wait to complete
    while (DataSz) {
        MasterWriteI2C3(i2cData[Index++]);
        IdleI2C3(); //Wait to complete
        DataSz--;
        //ACKSTAT is 0 when slave acknowledge,
        //if 1 then slave has not acknowledge the data.
        if (I2C3STATbits.ACKSTAT)
            break;
    }

    //now send a start sequence again
    RestartI2C3();  // Send the Restart condition
    IdleI2C3();     // Wait to complete
    MasterWriteI2C3((I2cAddress << 1) | 1); //transmit read command
    IdleI2C3();     // Wait to complete
    msb_columbCNT = MasterReadI2C3(); // read high byte
    IdleI2C3();     // Wait to complete
    AckI2C3();
    IdleI2C3();     // Wait to complete
    lsb_columbCNT = MasterReadI2C3(); // read low byte
    IdleI2C3();     // Wait to complete
    NotAckI2C3();
    IdleI2C3();     // Wait to complete
    accumulateCharge.v[0] = lsb_columbCNT;
    accumulateCharge.v[1] = msb_columbCNT;
    StopI2C3();     // Send the Stop condition
    IdleI2C3();     // Wait to complete

    return accumulateCharge.Val;
}

/*********************************************************************
 * Function:        void readI2Cstatus(void)
 *
 * PreCondition:    I2c must be configured and running
 *
 * Input:           None
 *
 * Output:         returns the status of the battery charge
 *
 *
 * Side Effects:
 *
 * Overview:        This function aquires the status bits
 *                     A[5]  Indicates accumulated charge hit either
 *                           top or bottom
 *                     A[3]  Indicates that the accumulated
 *                           charge value exceeded the
 *                           charge threshold high limit.
 *                     A[2]  Indicates that the accumulated
 *                           charge value dropped below
 *                           the charge threshold low limit
 *                     A[1]  voltage (at SENSE–) dropped
 *                           below selected VBAT threshold
 *                     A[0]  Indicates recovery from
 *                           undervoltage. If equal to 1,
 *                           a UVLO has occurred and
 *                           the content of registers is
 *                           uncertain.
 *
 *
 * Note:
 ********************************************************************/
BYTE Battery_status(void) {
    char i2cData[5];
    char DataSz;
    int Index = 0;
    BYTE I2cAddress;
    BYTE status = 0xff;

    I2cAddress = 0b01100100;
    i2cData[0] = (I2cAddress << 1) | 0; // Device Address &WR
    i2cData[1] = GAS_GAUGE_STATUS;
    DataSz = 2;

    IdleI2C3();    // Wait to complete
    StartI2C3();   // Send the Start Bit
    IdleI2C3();    // Wait to complete

    while (DataSz) {
        MasterWriteI2C3(i2cData[Index++]);
        IdleI2C3();                  // Wait to complete
        DataSz--;
        //ACKSTAT is 0 when slave acknowledge,
        //if 1 then slave has not acknowledge the data.
        if (I2C3STATbits.ACKSTAT)
            break;
    }
    RestartI2C3();                    // Send the Restart condition
    IdleI2C3();                       // Wait to complete
    i2cData[0] = (I2cAddress << 1) | 1;
    MasterWriteI2C3(i2cData[0]);      // transmit read command
    IdleI2C3();                       // Wait to complete
    status = MasterReadI2C3();
    IdleI2C3();
    if (I2C3STATbits.ACKSTAT)
        status = 0b01010101;
    NotAckI2C3();
    StopI2C3(); //Send the Stop condition
    IdleI2C3();
    return status;
}

/*********************************************************************
 * Function:        void Battery_Diagnostic(void)
 *
 * PreCondition:    I2c must be configured and running
 *
 * Input:           None
 *
 * Output:         None
 *
 *
 * Side Effects:
 *
 * Overview:  this function primarily helps determine battery discharge
 *            rate.  This rate is further used to set the coulomb level
 *            of the battery charge IC for low battery alert
 *
 * Note:
 *
 *
 ********************************************************************/
void Battery_Diagnostic(void) {
    unsigned int coulombCount;
    static int old_coulombCount = 99;
    BYTE status;
    BYTE chargerStatus;

    if (EXTERNAL_POWER_SENSE_PIN) {
        chargerStatus = chargerStatus_blocking();
        switch (chargerStatus) // inquire from charger
        {
            case DEFECTIVE_BATTERY:
                strcpy(pageInformation.instructions2, " Battery Defect ");
                break;
            case FAIL_TEMPERATURE:
                strcpy(pageInformation.instructions2, " Temp Fault");
                break;
            case PARTIAL_CHARGE:
                strcpy(pageInformation.instructions2, " Charging ");
                break;
            case FULL_CHARGE:
                strcpy(pageInformation.instructions2, " Battery Full ");
                break;
            default:
                strcpy(pageInformation.instructions2, " Battery Unknown ");
                break;
        }
    } else {
        strcpy(pageInformation.instructions2, " No external power ");
    }

    status = Battery_status() & 0b00011110;

    switch (status) // mask out the over and under flow
    {
        case BATTERY_GOOD:
            strcpy(pageInformation.instructions1, " Battery Good ");
            break;
        case BATTERY_CHARGE_ALERT_HIGH:
            strcpy(pageInformation.instructions1, "Charge ACC + ");
            break;
        case BATTERY_CHARGE_ALERT_LOW:
            strcpy(pageInformation.instructions1, "Charge ACC - ");
            break;
        case BATTERY_LOW_BATTERY:
            strcpy(pageInformation.instructions1, "Low bat voltage ");
            break;
        default:
            strcpy(pageInformation.instructions1, " Battery Unknown ");
            break;
    }

    coulombCount = read_gasGauge();
    if ((coulombCount != old_coulombCount)
            || (status != old_status)) {

        RTCCProcessEvents();
        old_coulombCount = coulombCount;
        old_status = status;

        pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_2);
        GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject, pageInformation.instructions2);
        GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);

        pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_1);
        GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject, pageInformation.instructions1);
        GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);

        sprintf((char *) &(pageInformation.instructions), (char*) "Coulomb: %6u ", coulombCount);
        pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS);
        GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject, pageInformation.instructions);
        GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);

        strcpy(pageInformation.instructions3, _time_str); // put time stamp when changed
        pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_3);
        GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject, pageInformation.instructions3);
        GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);

        update_display = GLOBAL_UPDATE;
    }
}

/*********************************************************************
 * Function:        BOOL clearGasGaugeAlert(void)
 *
 * PreCondition:    I2c must be configured and running
 *
 * Input:           None
 *
 * Output:         returns true if the status alert is cleared for
 *                 this device.
 *
 * Side Effects:
 *
 * Overview:
 *
 *
 * Note: If transfer is successfully completed, the LTC2941
 * will stop pulling down the AL/CC pin and will not respond to
 * further ARA requests until a new alert event occurs.
 ********************************************************************/
BOOL clearGasGaugeAlert(void) {
    BYTE ARA_address;
    BYTE ARP_Address;
    ARP_Address = 0b00001100;

    StartI2C3(); // Send the Start Bit
    IdleI2C3();  // Wait to complete

    MasterWriteI2C3((ARP_Address << 1) | 1); //transmit read command
    IdleI2C3();  // Wait to complete

    ARA_address = MasterReadI2C3(); // read byte 0b01100100;

    IdleI2C3(); // Wait to complete
    StopI2C3(); // Send the Stop condition
    IdleI2C3(); // Wait to complete 

    if (0b1100101 == ARA_address)
        return TRUE;
    return FALSE;
}

/*********************************************************************
 * Function:        BYTE monitorBatteryLevel(void)
 *
 * PreCondition:    I2c 1 must be configured and running
 *
 * Input:           None
 *
 * Output:          .
 *                 .
 *
 * Side Effects:
 *
 * Overview:        This function 
 *                  
 *
 *
 * Note:
 ********************************************************************/
void monitorBatteryLevel(void) {
    static DWORD timetick_Ms = 0;
    unsigned int coulombCNT;
    unsigned int count;
    BYTE status;
    if (EXTERNAL_POWER_SENSE_PIN == 0) {
        if (screenState != SCREEN_BATTERY) {
            if (GasGauge_alert_input == 0) {
                status = Battery_status();
                if (status & (CHARGE_ALERT_LOW | VBAT_ALERT)) {
                    BatteryLevel = 0;
                    screenState = SCREEN_POWER_DOWN_BATTERY_LOW;
                }
                // give warning and power down
                // change menu states to show warning
            }

            if (tick > (timetick_Ms + 10000)) {
                timetick_Ms = tick;

                coulombCNT = read_gasGauge();
                count = 59535 - coulombCNT;      // normalize battery value
                BatteryLevel = 255 - count / 233;
                if (coulombCNT < 6000) {
                    screenState = SCREEN_POWER_DOWN_BATTERY_LOW;
                    // give warning and power down
                    // change menu states to show warning

                }
            }
        }  // check for running battery test
    } // check for external power
}

/****************************************************************************
  Function:
    void ShowScreenBatteryDiagnostic( void )

  Description:
    This function erases the currently displayed screen and replaces it with
    a screen consisting of:
 * the Microchip icon
 * a one line title
 * 3 line of instructions

    The information for the screen contents is obtained from the global
    pageInformation structure.

  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Remarks:
    The global list of graphics objects is destroyed and replaced.  Therefore,
    this function cannot be called during the message processing of
    GOLMsgCallback().  It must be called during GOLDrawCallback() processing.
 ***************************************************************************/
void ShowScreenBatteryDiagnostic(void) {
    strcpy(pageInformation.title, "BATTERY ");
    InitializeScreen(); // set up logo and title text and destroys link list
    // of objects

    strcpy(pageInformation.instructions2, "Please wait ");
    strcpy(pageInformation.instructions1, "checking battery ");
    strcpy(pageInformation.instructions, "condition");

    strcpy(pageInformation.instructions3, "time stamp");
    strcpy(pageInformation.buttonTextBottomLeft, " OK");

    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS_2,
            0, 21, 127, 21 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10) + 4,
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions2,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);

    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS_1,
            0, 34, 127, 34 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10) + 4,
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions1,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);


    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS,
            0, 47, 127, 47 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10) + 4,
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);


    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS_3,
            0, 61, 127, 61 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10) + 4,
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions3,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);


    GFX_GOL_ButtonCreate(ID_BUTTON_GENERIC_HOME, // ID
            49, 81, 74, 95, 2,                // dimension and radius
            GFX_GOL_BUTTON_DRAW_STATE ,       // will be displayed after creation
            NULL,                             // bitmap
            NULL,                             // no release image used
            (GFX_XCHAR*) pageInformation.buttonTextBottomLeft, // text
            GFX_ALIGN_HCENTER | GFX_ALIGN_VCENTER,
            &pScheme_button); //



}

/****************************************************************************
  Function:
    bool ProcessMessage_BATTERY_test( WORD translatedMsg, GFX_GOL_OBJ_HEADER* pObj,
                        GFX_GOL_MESSAGE* pMsg )

  Description:
    This function processes the messages for the Battery test selection screen.  The
    global screenState is updated to proceed to the user's selection.  right
    key exits the test.  I could not use the home key to exit.

  Precondition:
    Call ShowScreenBatteryDiagnostic() prior to using this function to display the
    correct screen.

  Parameters:
    WORD translatedMsg  - The translated control-level message
    GFX_GOL_OBJ_HEADER* pObj    - Object to which the message applies
    GFX_GOL_MESSAGE* pMsg       - The original system message information

  Return Values:
    0   - Do not call the default message handler.
    1   - Call the default message handler.

  Remarks:
    global change variable to used  clean up later.
 ***************************************************************************/
bool ProcessMessage_BATTERY_test(WORD translatedMsg, GFX_GOL_OBJ_HEADER* pObj, GFX_GOL_MESSAGE* pMsg) {
    WORD controlID;
    controlID = GFX_GOL_ObjectIDGet(pObj);
    switch (controlID) {
        case ID_BUTTON_GENERIC_HOME:
            if (translatedMsg == GFX_GOL_BUTTON_ACTION_PRESSED) {
                screenState = SCREEN_DISPLAY_MAIN;
                return false; // Hidden button

            } else {
                return false;
            }
            break;
        default:
            return false;
    }

    return true; // Call the default handler to show the button state.
}

/****************************************************************************
  Function:
    void ShowScreenBatteryAlert( void )

  Description:
    This function erases the currently displayed screen and replaces it with
    a screen consisting of:
 * the Microchip icon
 * a one line title
 * 3 line of instructions

    The information for the screen contents is obtained from the global
    pageInformation structure.

  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Remarks:
    The global list of graphics objects is destroyed and replaced.  Therefore,
    this function cannot be called during the message processing of
    GOLMsgCallback().  It must be called during GOLDrawCallback() processing.
 ***************************************************************************/
void ShowScreenBatteryAlert(void) {
    strcpy(pageInformation.title, "Battery Alert !");
    InitializeScreen(); // set up logo and title text and destroys link list
    // of objects

    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS_2,
            0, 21, 127, 21 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_12) + 4,
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions2,
            GFX_ALIGN_CENTER,
            &VB12_Scheme);

    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS_1,
            0, 38, 127, 38 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_12) + 4,
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions1,
            GFX_ALIGN_CENTER,
            &VB12_Scheme);


    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS,
            0, 55, 127, 55 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_12) + 4,
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions,
            GFX_ALIGN_CENTER,
            &VB12_Scheme);
}

/****************************************************************************
  Function:
   void PowerDownSequence( void )

  Description:
    This function initializes the radio when powered on.  Shows hardware version
 software version, company name, battery condition, unit id, pan id, channel

  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Remarks:
    None
 ***************************************************************************/
void PowerDownSequence(void) {

    static int elapsedSeconds = 0;

    if (tick > tick_10ms + 100) {
        switch (elapsedSeconds) {
            case 0:
                ShowScreenBatteryAlert(); // create text objects
                break;

            case 1:
                power_mode(HIGH_POWER);
                strcpy(pageInformation.instructions2, " please charge ");
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_2);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject, pageInformation.instructions2);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                strcpy(pageInformation.instructions1, " battery low ");
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_1);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject, pageInformation.instructions1);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                strcpy(pageInformation.instructions, "turning off");
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject, pageInformation.instructions);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                update_display = GLOBAL_UPDATE;
                break;
            case 3:
                powerDown();
                break;
        }
        tick_10ms = tick;
        elapsedSeconds++;
    }
}

/****************************************************************************
  Function:
    WORD ShowScreenUserPowerDown(void)

  Description:
    This function sets up the screen to show the power down messages

  Precondition:
    Initialize display hardware

  Parameters:
    none

  Return Values:
    none

  Remarks:

 ***************************************************************************/
void ShowScreenUserPowerDown(void) {

    if ((TRAP_FAULT > 5) || (reset_reason == 8))TRAP_FAULT = 5; //none;
    strcpy(pageInformation.instructions2, "Reset ");
    strcat(pageInformation.instructions2, reset_codes[reset_reason]);
    strcat(pageInformation.instructions2, "  ");
    strcat(pageInformation.instructions2, "Trap ");
    strcat(pageInformation.instructions2, INT_codes[TRAP_FAULT]);

    TRAP_FAULT = 5;                           // Set to none
    pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_2);
    GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions2);
    GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);

    strcpy(pageInformation.title, "Power Down");
    strcpy(pageInformation.instructions, "  EPD DEV ");
    sprintf((char *) &(pageInformation.instructions),
            (char*) "ID:%02x %02x %02x %02x %02x %02x",
            UID48[0], UID48[1], UID48[2], UID48[3], UID48[4], UID48[5]);
    strcpy(pageInformation.instructions1, VERSION_STRING);
    strncat(pageInformation.instructions1, HARDWARE_VERSION_STRING, 2);

    InitializeScreen(); // set up logo and title text and destroys link list
    // of objects


    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS_2,
            0, 21, 127, 21 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10),
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions2,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);

    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS_1,
            0, 34, 127, 34 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10),
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions1,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);

    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS,
            0, 47, 127, 47 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10),
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);
}

/****************************************************************************
  Function:
    char powerDown( void )

  Description:
   // power consumption after power down is   35 uA
   // vREG                               Iq = 30 uA
   // micro                                    1 uA
   // misc parts                               4 uA
 *
 * 

  Precondition:


  Parameters:
    None

  Returns:
    None

  Remarks:
 *   radio stack modified to allow powering down the radio even though
 *   ENABLE_SLEEP is not defined.  If ENABLE_SLEEP is defined, the PAN Coordinator
 *   will not transmit unicast messages to the device.  Radio stack code has some
 *   of the #define ENABLE_SLEEP commented.
 ***************************************************************************/
void powerDown(void) {
    RCONbits.SWDTEN = 0;
    gauge_power(OFF);    // saves  .071 mA       
    status_1_pin = 0;
    IFS1bits.CNIF = 0;
#ifdef MRF24J40
    PHY_WAKE = 0;
    // make a power management reset to ensure device goes to sleep
    PHYSetShortRAMAddr(WRITE_SOFTRST, 0x04);
    //;write the registers required to place the device in sleep
    PHYSetShortRAMAddr(WRITE_TXBCNINTL, 0x80);
    PHYSetShortRAMAddr(WRITE_RXFLUSH, 0x60);
    PHYSetShortRAMAddr(WRITE_SLPACK, 0x80);
    //    set_IO_lowPowerMode();
#else
    SetRFMode(RF_SLEEP); // saves 3.530 mA 
#endif
    
    DRV_SPI_Deinitialize(1);
     DRV_SPI_Deinitialize(2);
      DRV_SPI_Deinitialize(3);
      CloseI2C3();
    //AD1PCFGL = 0xFFFF;   // set all as digital 
    
    
    MKROE_power_off();   // PIN 1
   // POWER INPUT             // PIN 2
   // GND                   // PIN 3
   // SENSE                 // PIN 4
   // 
   MIKROBUS_PWM_TRIS = 0;    // PIN 5
   MIKROBUS_PWM_LAT = 0;
   MIKROBUS_INT_TRIS = 0;   //PIN 6
   MIKROBUS_INT_LAT = 0;
   MIKROBUS_CS_TRIS = 0;    //PIN 7
   MIKROBUS_CS_LAT = 0;
   
   MIKROBUS_AN_TRIS = 0;    //PIN 8
   MIKROBUS_AN_LAT = 0;
   MIKROBUS_RST_TRIS = 0;   // PIN 9
   MIKROBUS_RST_LAT = 0;
   
   MCHP_23K256_SDI_TRIS = 1;   //  PIN 10
   
   MIKROBUS_EXT_P5_TRIS = 0;  // PIN 11
   MIKROBUS_EXT_P5_LAT = 0;
   
   MCHP_23K256_SDO_TRIS = 0;  // PIN 12
   MIKROBUS_RX_TRIS = 0;    // PIN 13
   MIKROBUS_RX_LAT = 0;
   MCHP_23K256_SCK_TRIS = 0; // PIN 14
   MIKROBUS_TX_TRIS = 0;    // PIN 15
   MIKROBUS_TX_LAT  = 0;
   MIKROBUS_SDA2_TRIS = 0;  // PIN 16
   MIKROBUS_SDA2_LAT = 0;
   USB_D_PLUS_TRIS = 0;      // PIN 17
   USB_D_PLUS_LAT = 0;
   MIKROBUS_SCL2_TRIS = 0;  // PIN 18
   MIKROBUS_SCL2_LAT = 0;
   USB_D_MINUS_LAT = 0;     // PIN 19
   USB_D_MINUS_TRIS = 0;
   USB_ID_LAT = 0;          // PIN 20
   USB_ID_TRIS = 0;
  

   
   
   

    Sleep();
    asm("reset");
}

void set_IO_lowPowerMode(void) {
    /// trying to reduce sleep current from 80microamps to 40 ... no luck so far
    //    MCHP_23K256_CS_LAT  = 1;                  // cs is active low
    //    mac_ee = 1;                               // cs is active low
    //
    //
    //    DRV_EPD_DischargeReset();
    //    DRV_EPD_SPI_CS_LAT     = 0;               // active low but should set to 0
    //    DRV_EPD_BorderDisable();                  // since this part is off
    //    //
    //    USB_POWER_CTL_EXT_pin = USB_POWER_CTL_EXT_OFF; // turn off 5 v regulator
    //
    //
    //
    //    //-----------------
    //    //  20 pin connector
    //    //-----------------
    //
    //    BLUETOOTH_EVENT_TRIS = OUTPUT_PIN;
    //    BLUETOOTH_EVENT_LAT = 0;
    //
    //    BLUETOOTH_CONNECT_TRIS = OUTPUT_PIN;
    //    BLUETOOTH_CONNECT_LAT = 0;
    //
    //    BLUETOOTH_CMD_MLDP_TRIS = OUTPUT_PIN;
    //    BLUETOOTH_CMD_MLDP_LAT = 0;
    //
    //    BLUETOOTH_CMD_MLDP_TRIS = OUTPUT_PIN;
    //    BLUETOOTH_CMD_MLDP_LAT  = 0;
    //
    //    BLUETOOTH_TX_TRIS = OUTPUT_PIN;
    //    BLUETOOTH_TX_LAT = 0;
    //
    //    BLUETOOTH_RX_TRIS = OUTPUT_PIN;
    //    BLUETOOTH_RX_LAT  = 0;
    //
    //    BLUETOOTH_CTS_TRIS = OUTPUT_PIN;
    //    BLUETOOTH_CTS_LAT  = 0;
    //
    //    BLUETOOTH_WAKEHW_TRIS   = OUTPUT_PIN;
    //    BLUETOOTH_WAKEHW_LAT = 0;
    //
    //    RB2_Pin6_TRIS   = OUTPUT_PIN;
    //    RB2_Pin6_LAT   = 0;
    //
    //
    //
    //    SPI1STATbits.SPIEN = 0;  // disable radio spi module
    //    SPI1CON1bits.DISSCK = 1;  //SPIx clock on SCKx pin is disabled; pin functions as I/O
    //    SPI1CON1bits.DISSDO = 1;  //SDOx pin is not used by the module; pin functions as I/O
    //    SDI_TRIS = 1;
    //    SDO_TRIS = 1;
    //    SCK_TRIS = 1;
    //    SPI2STATbits.SPIEN = 0;  // disable radio spi module
    //    SPI2CON1bits.DISSCK = 1;  //SPIx clock on SCKx pin is disabled; pin functions as I/O
    //    SPI2CON1bits.DISSDO = 1;  //SDOx pin is not used by the module; pin functions as I/O
    //    MCHP_23K256_SCK_TRIS = 1;
    //    MCHP_23K256_SDO_TRIS = 1;
    //    MCHP_23K256_SDI_TRIS  = 1;
    //
    //
    //    SPI3STATbits.SPIEN = 0;  // disable radio spi module
    //    SPI3CON1bits.DISSCK = 1;  //SPIx clock on SCKx pin is disabled; pin functions as I/O
    //    SPI3CON1bits.DISSDO = 1;  //SDOx pin is not used by the module; pin functions as I/O
    //    DRV_EPD_SPI_SCK_TRIS = 1;
    //    DRV_EPD_SPI_SDO_TRIS = 1;
    //    DRV_EPD_SPI_SDI_TRIS  = 1;
    //
    //    //CloseI2C3();
    //    I2C3CONbits.I2CEN = 0;  //Disables the I2Cx module; all the I2C™ pins are controlled by port functions
    //    TRISEbits.TRISE7 = 1;  // set sda pin to input
    //    TRISEbits.TRISE6 = 1;
    //
    //    AD1CON1bits.ADON = 0;  //A/D Converter is disabled
    //    AD1PCFGbits.PCFG7 = 1; // Pin for corresponding analog channel is in Digital mode;

void powerDownIO(void) {

    AD1PCFGL = 0xFFFF;   // set all as digital 
    TRISB = 0xFF;
    TRISC = 0xFF;
    TRISD = 0xFF;
    TRISE = 0xFF;
    TRISF = 0xFF;
    TRISG = 0xFF;
    
DRV_EPD_DISPLAY_BUSY_TRIS = 1;                // input
    DRV_EPD_DISPLAY_RESET_TRIS = 0;
    //DRV_EPD_DISPLAY_PWM_TRIS = 0;               // not available in hw rev4
    DRV_EPD_DISPLAY_PANEL_ON_TRIS = 0;
    DRV_EPD_DISPLAY_DISCHARGE_TRIS = 0;
   // DRV_EPD_DISPLAY_BORDER_TRIS = 0;
    DRV_EPD_DISPLAY_TEMP_TRIS = 1;
    DRV_EPD_SPI_CS_TRIS     = 0;
   
   
    DRV_EPD_PowerDisable();

  
 

    //-----------------
    //  WIRELESS
    //-----------------


    Data_nCS_TRIS = OUTPUT_PIN;
    Config_nCS_TRIS = OUTPUT_PIN;
    PHY_RESETn_TRIS = OUTPUT_PIN;
    Data_nCS = 1;
    Config_nCS = 1;
    PHY_RESETn = 0;

    //-----------------
    //  MEMORY
    //-----------------
    MCHP_23K256_CS_TRIS = OUTPUT_PIN;
    MCHP_23K256_CS_LAT  = 1;                  // cs is active low
    mac_ee = 1;                               // cs is active low
    mac_ee_TRIS = OUTPUT_PIN;

    DRV_EPD_DISPLAY_RESET_LAT = 0;
    DRV_EPD_DischargeReset();
    DRV_EPD_SPI_CS_LAT     = 0;               // active low but should set to 0
   
    USB_POWER_CTL_EXT_pin = USB_POWER_CTL_EXT_OFF; // turn off 5 v regulator
   

    PHY_IRQ1 = 0;                 // clear interrupt flags
    PHY_IRQ0 = 0;
    PHY_IRQ1_En = 1;              // enable interrupts
    PHY_IRQ0_En = 1;

    //-----------------
    //  A/D CONVERTER
    //-----------------
    initialize_temperature();


    
    MIKROBUS_INT_TRIS = INPUT_PIN;
    MIKROBUS_INT_LAT = 0;

    MIKROBUS_EXT_P5_TRIS = OUTPUT_PIN;
    MIKROBUS_EXT_P5_LAT  = 0;

    MIKROBUS_SCL2_TRIS = INPUT_PIN; //
    MIKROBUS_SCL2_LAT = 0;

    MIKROBUS_SDA2_TRIS = OUTPUT_PIN;
    MIKROBUS_SDA2_LAT  = 0;

    MIKROBUS_RST_TRIS = OUTPUT_PIN;
  //  MIKROBUS_RST_LAT = 0;

    MIKROBUS_TX_TRIS = OUTPUT_PIN;
    MIKROBUS_TX_LAT = 0;

    MIKROBUS_RX_TRIS = INPUT_PIN;
    BLUETOOTH_EVENT_TRIS = INPUT_PIN;
    
    MIKROBUS_CS_TRIS = INPUT_PIN;
    
     MIKROBUS_CS_TRIS = INPUT_PIN;   // do not power the device through the I/o
//    IRQ_IN_Direction = INPUT_PIN;
//    SSI_0_Direction  = INPUT_PIN;
//    SSI_1_Direction  = INPUT_PIN;
    MIKROBUS_INT_TRIS = INPUT_PIN;
//    MCU_Init();
    
    


    //-----------------
    //  USB
    //-----------------
    EXTERNAL_POWER_SENSE_TRIS = INPUT_PIN; // external power detection 
    EXTERNAL_POWER_SENSE_PIN_cn_enable = 0;
    USB_D_PLUS_LAT = 0;
    USB_D_PLUS_TRIS = OUTPUT_PIN;

    USB_D_MINUS_LAT = 0;
    USB_D_MINUS_TRIS = OUTPUT_PIN;

    USB_ID_LAT = 0;
    USB_ID_TRIS = OUTPUT_PIN;

    //-----------------
    //  POWER
    //-----------------
    chrg_TRIS = INPUT_PIN;           // battery charger status
    chrg_input_weakPullUp = 1;       // enable weak pullup

    GasGauge_alert_TRIS = INPUT_PIN; // battery charge out of alert threshold
    GasGauge_weakPullUp = 1;         // open collector device needs pullup
    GasGauge_cn_enable = 0;          // firmware to poll the bit

    CloseI2C3();			   		 //Disable I2C3 module if enabled previously
    
            // start off with no alerts.
    EXTERNAL_POWER_SENSE_TRIS = INPUT_PIN; // sense for external 5v power

 

    status_1_pin = 0;             // turn off status led

 
    

    //-----------------
    //  SYSTEM CLOCK
    //-----------------
   
    T4CONbits.TON = 0; 
    // power conservation


    //-----------------
    //  INTERRUPTS
    //-----------------
    RCON = 0;                     // clear power on reset bit
    RCONbits.SWDTEN = 1;          // software watchdog enabled
    RCONbits.PMSLP = 0;           // PMSLP: Program Memory Power During Sleep bit
                                  // 1 = Program memory bias voltage remains 
                                  //  powered during Sleep.
                                  // 0 = Program memory bias voltage is powered 
                                  // down during Sleep and voltage regulator 
                                  // enters Standby mode.

}
}

/****************************************************************************
  Function:
    char userPowerDown( void )

  Description:
    This function changes the screenState to power down the device when
 *  S3 is pressed for a long duration

  Precondition:


  Parameters:
    None

  Returns:
    None

  Remarks:
 *
 ***************************************************************************/
void userPowerDown(void) {
    if (switch_duration == LONG_PRESS) {
        screenState = SCREEN_DISPLAY_USER_POWER_DOWN;
        switch_duration = SHORT_PRESS; // reset sticky bit
    }
}

/****************************************************************************
  Function:
    char chargerStatus_blocking( void )

  Description:
    This function counts the time the battery status line is high and low for
 * one second.  The processor is blocked to run other code while the status is
 * aquired. pulse duty cycle from the charger IC determines the battery status.

  Precondition:  Timer4 and Tick initialized and running


  Parameters:
    None

  Returns:
 *         PARTIAL_CHARGE
 *         FULL_CHARGE
 *                             FAIL_TEMPERATURE  not with this charger
 *         DEFECTIVE_BATTERY

  Remarks:
 *   this is blocking code.  be careful not to trip the WDT
 ***************************************************************************/
char chargerStatus_blocking(void) {
    char samplePeriod = 100; // sample for 1 second
    char lowPeriodCount = 0;
    char highPeriodCount = 0;
    char result;
    while (samplePeriod--) {
        __delay_ms(10);
        if (chrg_input)
            highPeriodCount++;
        else
            lowPeriodCount++;
    }
    if (highPeriodCount == 0)
        result = PARTIAL_CHARGE;
    else if (lowPeriodCount == 0)
        result = FULL_CHARGE;
    else if (lowPeriodCount > highPeriodCount) //25% duty cycle
        result = DEFECTIVE_BATTERY;
    else
        result = DEFECTIVE_BATTERY;
    return result;
}

/****************************************************************************
  Function:
    void external_power_applied( void )

  Description:
    This function is called when power is applied.  for safety reasons the
 * device does not operate when plugged into an external power source. operation
 * around a pool with 120vac could be dangerous.  this function updates the
 * power down message with the firmware version displayed

  Precondition:  


  Parameters:
    None

  Returns:
 

  Remarks:
 * call this function for charging mode only.  device will not 
 * respond to key presses
 * 

 ***************************************************************************/
void external_power_applied(void) {
    static char debounce_delay;
    static DWORD pollCounter;
    char CCstatus;
    static BOOL OK_to_reset_ChargerACC;
    static char CCstatus_old = 99;
    switch (chargerState) {
        case EXTERNAL_PWR_APPLIED:
            power_mode(HIGH_POWER);
            EXTERNAL_POWER_SENSE_PIN_cn_enable = 0;
            ShowScreenUserPowerDown();
            strcpy(pageInformation.title, "Ext. Power");
            strcpy(pageInformation.instructions2, "  Checking Status ");
            strcpy(pageInformation.instructions1,  VERSION_STRING);
            strcpy(pageInformation.instructions, " - - - - - ");

            // external power detected
            debounce_delay = 100;
            OK_to_reset_ChargerACC = false;


            chargerState = DEBOUNCE_PWR;
            break;
        case DEBOUNCE_PWR:
            // wait one second
            if (debounce_delay-- == 0)
                chargerState = READ_STATUS;
            break;
        case READ_STATUS:
            power_mode(HIGH_POWER);
            CCstatus = chargerStatus_blocking(); // blocks for 1 second
            pollCounter = tick;
            if (EXTERNAL_POWER_SENSE_PIN == 0) {
                strcpy(pageInformation.instructions2, "Power Removed ");
                chargerState = POWER_OFF;
            } else if (CCstatus == DEFECTIVE_BATTERY) {
                strcpy(pageInformation.instructions2, "Defective Battery ");
                chargerState = POWER_OFF;
            } else if (CCstatus == FAIL_TEMPERATURE) {
                strcpy(pageInformation.instructions2, "Temperature Fault");
                chargerState = POWER_OFF;
            } else if (CCstatus == PARTIAL_CHARGE) {
                strcpy(pageInformation.instructions2, "  Charging ");
                OK_to_reset_ChargerACC = true;
                chargerState = SLEEP;
            } else if (CCstatus == FULL_CHARGE) {
                strcpy(pageInformation.instructions2, "Charge Complete");
                //chargerState = POWER_OFF;
                chargerState = SLEEP;

            }
            if (CCstatus_old != CCstatus) {
                CCstatus_old = CCstatus;
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_2);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject, pageInformation.instructions2);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                update_display = GLOBAL_UPDATE;
            }
            break;
            // polling
        case SLEEP:

            if (tick > pollCounter + 500) {
                chargerState = READ_CHARGE_STATUS;
            }
            break;
        case READ_CHARGE_STATUS:
            power_mode(HIGH_POWER);
            CCstatus = chargerStatus_blocking(); // blocks for 1 second
            pollCounter = tick;
            if (EXTERNAL_POWER_SENSE_PIN == 0) {
                strcpy(pageInformation.instructions2, "Power Removed ");
                chargerState = POWER_OFF;
            } else if (CCstatus == DEFECTIVE_BATTERY) {
                strcpy(pageInformation.instructions2, "Defective Battery ");
                chargerState = POWER_OFF;
            } else if (CCstatus == FAIL_TEMPERATURE) {
                strcpy(pageInformation.instructions2, "Temperature Fault");
                chargerState = POWER_OFF;
            } else if (CCstatus == PARTIAL_CHARGE) {
                strcpy(pageInformation.instructions2, "  Charging ");
                OK_to_reset_ChargerACC = true; // only update if finished charging
                chargerState = SLEEP;
            } else if (CCstatus == FULL_CHARGE) {
                strcpy(pageInformation.instructions2, "Charge Complete");
                chargerState = SLEEP;  // 
                if (OK_to_reset_ChargerACC) {
                    resetChargeAccumulator(); // update registers
                    OK_to_reset_ChargerACC =  false;
                }
            }
            if (CCstatus_old != CCstatus) {

                CCstatus_old = CCstatus;
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_2);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject, pageInformation.instructions2);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                update_display = GLOBAL_UPDATE;
            }
            break;
        case POWER_OFF:
            screenState = SCREEN_POWER_DOWN;
            break;
        default:
            chargerState = EXTERNAL_PWR_APPLIED;
            break;
    }
}

/****************************************************************************
  Function:
    void pollExternalPower( void )

  Description:
 *    This function is polled and when external power is applied the application
 *    changes state and displays charging status.
 *
 *
 *
  Precondition:  Timer4 and Tick initialized and running

  Parameters:
    None

  Returns:
 *         sets screenState and chargerState global variables


  Remarks:
 *   this can be modified to allow for the device to operate while power is
 *   applied
 ***************************************************************************/
void pollExternalPower() {
    static char debounceCount;
    // filter out any noise on the line
    if (EXTERNAL_POWER_SENSE_PIN)
        debounceCount++;
    else
        debounceCount = 0;
    if ((screenState != SCREEN_EXTERNAL_POWER_APPLIED)
            & (screenState != SCREEN_POWER_DOWN)
            & (screenState != SCREEN_BATTERY))
        if (debounceCount > 100) {
            screenState = SCREEN_EXTERNAL_POWER_APPLIED;
            chargerState = EXTERNAL_PWR_APPLIED;
        }
}




/////////////////////////////////// to be deleted  ////////////////////////////



