

#include <stdlib.h>
#include <system.h>
#include "driver/uart/drv_uart1.h"
#include "Bluetooth.h"
#include <string.h>


uint8_t responseBuffer[25];



/****************************************************************************
  Function:
    send_uart_command(uint8_t *command)

  Description:
 *  this function sends \r\n terminated command strings to the RN4020 module
   
  Precondition:
 it is best if the txByteQ is empty, but if it is not empty the TasksTX() will 
 * be called until the buffer is empty 

  Parameters
 pointer to  \r\n terminated string 

  Returns:
 number of characters to be sent.

  Remarks:  this function loads data into the txByteQ buffer from data pointed 
 * to by the command pointer.  If the buffer becomes full the TasksTX() function
 * will send out data to the tx fifo until the buffer is empty. 
     
 ***************************************************************************/
int send_uart_command(uint8_t *command) {
    int numBytes = 0;
    int writebufferLen = strlen((char *) command);
    DRV_UART1_TRANSFER_STATUS status ;
    while ( numBytes < writebufferLen) {
        status = DRV_UART1_TransferStatus() ;
        DRV_UART1_TasksError();
        if (status & DRV_UART1_TRANSFER_STATUS_TX_EMPTY)
            numBytes += DRV_UART1_Write ( (uint8_t *) (command + numBytes), writebufferLen - numBytes ) ;

        DRV_UART1_TasksTX();// chunk out at most 4 bytes of data from txByteQ 
    }                       // buffer to the txFiFo. this will clear space in the
    return numBytes;        // case that the txByteQ is not empty
}

void transferCommand(void) {
    while (!(DRV_UART1_TransferStatus() & DRV_UART1_TRANSFER_STATUS_TX_EMPTY)) {
        DRV_UART1_TasksTX();
        DRV_UART1_TasksRX();  // prevent over run errors
    }
}

/****************************************************************************
  Function:
    bool receive_uart_response( uint8_t timeout)

  Description:
 *  this function waits for a linefeed terminated response 
   
  Precondition:
 

  Parameters
 uint8_t timeout  time in ms to wait for a linefeed (\n) terminated response string 

  Returns:
 true if response with linefeed, false otherwise

  Remarks: if (\n) is found the data from the rxByteQ is moved to the 
 * responseBuffer
     
 ***************************************************************************/
bool receive_uart_response( uint8_t timeout) {
    DWORD tick_10ms_time_out;

    tick_10ms_time_out = tick + timeout;
    while (tick < tick_10ms_time_out ) {
        DRV_UART1_TasksRX();
        if (getResponse()) {
            return true;
        }
    }
    return false;
}


/****************************************************************************
  Function:
    bool getResponse(void)

  Description:
 *  this function peeks into the rxByteQ looking for \n.  it the newline character
 * is detected the data is moved from the rxByteQ to the responseBuffer 
   
  Precondition:
 * should be expecting a response from a transmitted command
 

  Parameters
 none

  Returns:
 true if response with linefeed, false otherwise

  Remarks: if (\n) is found the data from the rxByteQ is moved to the 
 * responseBuffer
     
 ***************************************************************************/
bool getResponse(void) {
    unsigned int size, k;

    uint16_t i;
    bool receivedCommand_status = false;
    if (!DRV_UART1_RXBufferIsEmpty()) {
        size = DRV_UART1_RXBufferSizeGet();
        for (i = 0; i < size; i++) {
            if (DRV_UART1_Peek(i) == '\n') {
                for (k = 0; k < 25; k++)responseBuffer[k] = 0;
                receivedCommand_status = true;
                DRV_UART1_Read( responseBuffer ,  size);
               // DRV_UART1_Read( responseBuffer ,  i);
                break;
            }
        }
    }
    return receivedCommand_status;
}

/****************************************************************************
  Function:
    bool expect(command,expect_string,expect_stringLength,trycount)

  Description:
    This function sends a command and waits 2 seconds for a response if the 
 * response matches the expect string the function returns true. the function
 * will attempt for a true condition with the attempt count value. 

  Precondition:
     serial port must be configured,tick timer, and power applied to module,

  Parameters:
    uint8_t * command       send command to serial port
    uint8_t * expectString  compare to response
    uint8_t size            expectString length
    uint8_t attempt         number of tries for true return
 * 
  Returns:
     True if expectString matches response, false otherwise.

  Remarks:
 * should add timeout parameter to function

 ***************************************************************************/
bool expect(uint8_t * command, uint8_t * expectString, uint8_t size, uint8_t attempt) {
    bool result =  false;
    uint8_t trycounter;
    for (trycounter = 0; trycounter < attempt; trycounter++) {
        send_uart_command(command);
        transferCommand();
        if (receive_uart_response(200)) {
            if (strncmp((char *) responseBuffer, (char *) expectString, size ) == 0) {
                result =  true;  // matches expect string
                break;
            }
        }
    }
    return result;
}


char process_BT_connection_commands(char *commandBuffer) {
    char bt_status = 99;

    if (strncmp(commandBuffer, "ERR", 3) == 0)
        bt_status = 0;
    else if (strncmp(commandBuffer, "AOK", 3) == 0)
        bt_status = 1;
    else if (strncmp(commandBuffer, "Reboot", 6) == 0)
        bt_status = 2;
    else if (strncmp(commandBuffer, "CMD", 3) == 0)
        bt_status = 3;
    else if (strncmp(commandBuffer, "Connected", 9) == 0)
        bt_status = 4;
    else if (strncmp(commandBuffer, "Connection End", 10) == 0)
        bt_status = 5;
    else
        bt_status = 98;

    return bt_status;
}

char process_BT_Application_commands(char *commandBuffer) {
    char bt_app_status = 99;

    if (strncmp(commandBuffer, "Reset", 3) == 0)
        bt_app_status = 0;
    else if (strncmp(commandBuffer, "Status", 6) == 0)
        bt_app_status = 1;
    else if (strncmp(commandBuffer, "Power Down", 6) == 0)
        bt_app_status = 2;
    else if (strncmp(commandBuffer, "EchoMode", 8) == 0)
        bt_app_status = 3;
    else if (strncmp(commandBuffer, "ToggleLed", 9) == 0)
        bt_app_status = 4;
    else
        bt_app_status = 98;

    return bt_app_status;
}

void flushRXbuffer( uint8_t * buffer, uint8_t size) {
    uint8_t eachArrayElement;
    DRV_UART1_Flush_RX_buffer();
    for (eachArrayElement = 0; eachArrayElement < size; eachArrayElement++) {
        *buffer = 0;  // fill array with nul 
        buffer++;
    }
}

/****************************************************************************
  Function:
    configure_Bluetooth_MLDP(void)

  Description:
    This function configures the RN4020 for the following:
 *  Supported Features SR register
 * Auto Advertise
 * Enable MLDP
 * UART Flow Control
 * Enable Remote Command
 * Auto-enter MLDP Mode
 *  Services Supported SS register
 * Device Information
 * Battery
 * verifies firmware version 1.20

  Precondition:
     serial port must be configured, use usb 5 volt power rail,

  Parameters:
    None

  Returns:
    None

  Remarks:
 * if the expect command fails the led will rapidly flash forever. If wdt is 
 * enabled the cpu will reset.
 * 

 ***************************************************************************/
bool configure_Bluetooth_MLDP(void) {
    bool expect_result;
    uint8_t reset[7] = "r,1\r\n";
    uint8_t SR_command[15] = "sr,32200800\r\n";
    uint8_t SS_command[15] = "ss,c0000000\r\n";
    uint8_t SP_command[7] = "sp,7\r\n";
    uint8_t VER_response[16] = "MCHP BTLE v1.20";
    uint8_t version_command[4] = "V\r\n";
    //uint8_t connection_command[4] = "Q\r\n";

    uint8_t AOK_resonse[4]  = "AOK";  // include nul  character

    USB_POWER_CTL_EXT_pin = USB_POWER_CTL_EXT_OFF; // make sure it is off
    // clear all inputs to the module to prevent latch up when power is applied
    BLUETOOTH_RTS_LAT = 0;  //UxRTS pin is asserted (driven low) whenever the receiver is ready to receive data
    BLUETOOTH_WAKESW_LAT = 0;
    BLUETOOTH_WAKEHW_LAT = 0;
    BLUETOOTH_CMD_MLDP_LAT = 0;
    BLUETOOTH_TX_LAT = 0;



    // apply power
    USB_POWER_CTL_EXT_pin = USB_POWER_CTL_EXT_ON;
    
    U1STAbits.UTXEN = 1;

    BLUETOOTH_WAKESW_LAT = 1;   // When WAKE_SW is set high, the module wakes up 
    // and is set into Active mode.
    BLUETOOTH_WAKEHW_LAT = 1;   // Setting WAKE_HW (pin 15) high wakes the RN4020 
    // module from Dormant mode.
    BLUETOOTH_CMD_MLDP_LAT = 0; // CMD/MLDP must be set low so that the RN4020 
    // module is returned to Command mode


    delay_ms(500); // give time for 5volt rail to power up
    flushRXbuffer( responseBuffer, 25); // start from a clear buffer state

    expect_result = expect(version_command, VER_response, 15, 2);
    if (expect_result == false) return false; //error_blink(50000);

    expect_result = expect(SS_command, AOK_resonse, 3, 2);
    if (expect_result == false) return false; //error_blink(50000);

    expect_result = expect(SR_command, AOK_resonse, 3, 1);
    if (expect_result == false)return false;  //error_blink(50000);

    expect_result = expect(SP_command, AOK_resonse, 3, 1);
    if (expect_result == false) return false; //error_blink(50000);

    send_uart_command(reset);    // reset module
    transferCommand();
    delay_ms(100);

    BLUETOOTH_CMD_MLDP_LAT = 1; // Once MLDP mode is entered by setting CMD/MLDP
    // high, all data from the UART is sent to the peer device as a data stream.
    return true;
}


void Bluetooth_disconnect(void){
   
    BLUETOOTH_WAKESW_LAT = 0;   // When WAKE_SW is set high, the module wakes up 
    // and is set into Active mode.
    BLUETOOTH_WAKEHW_LAT = 0;   // Setting WAKE_HW (pin 15) high wakes the RN4020 
    // module from Dormant mode.
    BLUETOOTH_CMD_MLDP_LAT = 0; // CMD/MLDP must be set low so that the RN4020 
    // module is returned to Command mode
    
   // BLUETOOTH_TX_LAT = 0;
    U1STAbits.UTXEN = 0;
    
    USB_POWER_CTL_EXT_pin = USB_POWER_CTL_EXT_OFF;
     
}

/****************************************************************************
  Function:
    error_blink(void)

  Description:
    This function is called to indicate an error condition.  It does not require
 * any timer resources.  

  Precondition:
     none

  Parameters:
    None

  Returns:
    None

  Remarks:
 * if the expect command fails the led will rapidly flash forever. If wdt is 
 * enabled the cpu will reset.
 * 
 ***************************************************************************/
void error_blink(uint16_t blink) {
    uint16_t count;
    uint16_t offCount;
    while (1) {
        if (count++ > blink) {
            count = 0;
            status_1_pin ^= 1; // toggle 
            if (status_1_pin == 0 )
                for (offCount = 0; offCount < 15000 ; offCount ++) {
                    Nop();
                    Nop();
                    Nop();
                }
        }
    }
}