/********************************************************************
 ********************************************************************
 * FileName:    drv_spi_THUNDER.c
 * Dependencies:
 * Processor:	 PIC24
 * Complier:     Microchip xc16 ver 1.26
 * Company:      Electronic Design Solutions, LLC.
 *
 *********************************************************************
 * File Description:
 *
 *  * Description:
 This file contains the low level drivers for the Lightning board
 *
 * Change History:
 *  Rev   Date         Author    Description
 *  1.0   3/23/2016    DRW      Initial revision
 ********************************************************************/
/********************************************************************
Software License Agreement:

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *******************************************************************/

#include <stdint.h>
#include "system.h"
#include "drv_spi_THUNDER.h"
#include "string.h"

static DRV_SPI_INIT_DATA thunderInitData;

/************************************************************************
 * Macros
 ************************************************************************/
#define SPILOCK(ch)                         DRV_SPI_Lock(ch)
#define SPIUNLOCK(ch)                       DRV_SPI_Unlock(ch)
#define SPIINITIALIZE(initData)             DRV_SPI_Initialize(initData);
#define PUTSPIBYTE(ch, byte)                {                           \
                                                DRV_SPI_Put(ch, byte);  \
                                            }

#define GETSPIBYTE(ch, data)                {                           \
                                                data = DRV_SPI_Get(ch); \
                                            }

/******************************************************************************
  Function:
    void DRV_LIGHTINING_Initialize( const SYS_MODULE_INDEX index,
                                           const SYS_MODULE_INIT * const init )

  Summary:
    Initializes hardware and data for the given instance of the lightning module

  Description:
    This routine initializes hardware for the instance of the lightning module,
    using the hardware initialization given data.  It also initializes all
    necessary internal data.

  Parameters:
    pInitData - Pointer to the data structure containing all data
                necessary to initialize the hardware. This pointer may
                be null if no data is required and static initialization
                values are to be used.

  Returns:
    None
 ******************************************************************************/
void DRV_LIGHTNING_Initialize(DRV_SPI_INIT_DATA *pInitData) {
    SPIINITIALIZE( pInitData);
    memcpy(&thunderInitData, pInitData, sizeof (DRV_SPI_INIT_DATA));
}

/******************************************************************************
  Function:
    void DRV_Thunder_Write(unint8_t address, uint8_t data1) 

  Summary:
    Writes commands to the device.
    
  Description:
    This routine unlocks the spi port, drives the chip select low,sends 
 *  the address byte,and data
 * sequence

  Parameters:
 *    address
 *    data

  Returns:
    None
 ******************************************************************************/
void DRV_Thunder_Write(uint8_t address, uint8_t data1)  {

    address &= 0b00111111;  // clear bits 6 and 7

    while (!SPILOCK(thunderInitData.channel));

    DRV_SPI_Initialize( (DRV_SPI_INIT_DATA *) & thunderInitData);

    THUNDERCSLow();
    PUTSPIBYTE(thunderInitData.channel, address);
    PUTSPIBYTE(thunderInitData.channel, data1);
    THUNDERCSHigh();
    SPIUNLOCK(thunderInitData.channel);
}

/******************************************************************************
  Function:
    uint8_t DRV_Thunder_read(unint8_t address) 

  Summary:
    reads byte from Address.
    
  Description:
    This routine unlocks the spi port, drives the chip select low,sends 
 *  the address byte,and reads the data
 

  Parameters:
 *    address
 

  Returns:
    single 8 bit data
 ******************************************************************************/
uint8_t DRV_Thunder_Read(uint8_t address) {
    uint8_t tmp = 0;

    while (!SPILOCK(thunderInitData.channel));

    DRV_SPI_Initialize( (DRV_SPI_INIT_DATA *) & thunderInitData);
    // set bits for read
    address &= 0b01111111;  // clear bit 7
    address |= 0b01000000;  // set bit 6

    THUNDERCSLow();
    PUTSPIBYTE(thunderInitData.channel, address);
    GETSPIBYTE(thunderInitData.channel, tmp);
    THUNDERCSHigh();
    SPIUNLOCK(thunderInitData.channel);

    return tmp;
}



/******************************************************************************
  Function:
    uint8_t DRV_Thunder_Read_distance(void) 

  Summary:
    Function reads distance from detected thunder in Km units
    
  Description:
 

  Parameters:
 *    none
 

  Returns:
    single 8 bit data
 ******************************************************************************/
uint8_t DRV_Thunder_Read_distance(void) {
    uint8_t Out_thunder_distance;

    Out_thunder_distance = DRV_Thunder_Read(0x07) & 0x3F;

    return Out_thunder_distance;
}


/******************************************************************************
  Function:
    uint32_t DRV_Thunder_Read_Energy(void) 

  Summary:
    Function reads energy of detected thunder
    
  Description:
 

  Parameters:
 *    none
 

  Returns:
    single 32 bit energy data
 ******************************************************************************/
uint32_t DRV_Thunder_Read_Energy(void) {
    uint8_t low_byte, mid_byte;
    uint32_t Out_thunder_energy;

    Out_thunder_energy = DRV_Thunder_Read(0x06) & 0x1F;
    mid_byte = DRV_Thunder_Read(0x05);
    low_byte = DRV_Thunder_Read(0x04);

    Out_thunder_energy = (Out_thunder_energy << 8);
    Out_thunder_energy = (Out_thunder_energy | mid_byte);
    Out_thunder_energy = (Out_thunder_energy << 8);
    Out_thunder_energy = (Out_thunder_energy | low_byte);

    return Out_thunder_energy;
}

/*******************************************************************************
 * Function Thunder_Init()
 * ------------------------------------------------------------------------------
 * Overview: Function Initializes Thunder chip
 * Input: register address, data
 * Output: Nothing
 *******************************************************************************/
void DRV_Thunder_Init(void) {
    unsigned short temp;
    THUNDERCSHigh();
    
    DRV_THUNDER_POWER_UP();
    
    DRV_Thunder_Write(0x3C, 0x96);           // set all registers in default mode
    DRV_Thunder_Write(0x3D, 0x96);           //Calibrates automatically the internal RC Oscillators

    
//    AFE_GB    [5:1]  AFE Gain Boost
//    PWD       [0]    Power-down
    temp = DRV_Thunder_Read(0x00) & 0xC1;
    DRV_Thunder_Write(0x00, ((_INDOOR << 1) | temp)); // set to indoor
//-----------------------------------------------------------------------------

// The output signal of the AFE is monitored by the watchdog, which enables 
//the signal validation (see Signal Verification on page 11) in case the
//input signal crosses a certain threshold. The AS3935 is automatically set 
//back to Listening Mode once the Signal Validation block has made an
//assessment on the nature of the received signal (lighting or disturber). 
//With register REG0x01[3:0] it is possible to change the level of this
//threshold to increase the robustness to disturbers. If higher thresholds 
//are used, the AS3935 would loose sensitivity for very far lightning events,
//with an improvement of the man-made disturber rejection as benefit.
    
//    NF_LEV       [6:4]   Noise Floor Level
//    WDTH         [3:0]   Watchdog threshold
    
//                 Continuous Input Noise Level
//
//[μVrms] (outdoor) [μVrms] (indoor) REG0x01[6] REG0x01[5] REG0x01[4]
//----------------  ---------------  ---------- ---------- ----------
//  390              28                  0          0          0
//  630              45                  0          0          1
//  860              62                  0          1          0
//  1100             78                  0          1          1
//  1140             95                  1          0          0
//  1570            112                  1          0          1
//  1800            130                  1          1          0
//  2000            146                  1          1          1
    
    temp = DRV_Thunder_Read(0x01) & 0x80;
    DRV_Thunder_Write(0x01, 0x44 | temp);    // set NFL and WDTreshold
//------------------------------------------------------------------------------
    
//it is possible to allow the AS3935 to issue lightning interrupts only if a 
//minimum number of events (lightning) have been detected in the last 15 
//minutes. The minimum number of lightning events can be set with register 
//REG0x02[5:4].
    
//    Minimum Number of Lightning  REG0x02[5]  REG0x02[4]
//    ---------------------------  ----------  ----------
//               1                   0            0
//               5                   0            1
//               9                   1            0
//              16                   1            1
    
    
//    CL_STAT      [6]    Clear statistics
//    MIN_NUM_LIGH [5:4]  Minimum number of lightning
//    SREJ         [3:0]  Spike rejection
    
    temp = DRV_Thunder_Read(0x02) & 0x80;    // clear statistics, min number of ligtning, spike rejection
    DRV_Thunder_Write(0x02, 0x40 | temp);
//------------------------------------------------------------------------------
//    LCO_FDIV     [7:6]  Frequency division ration for antenna tuning
//    MASK_DIST    [5]    Mask Disturber
//    INT          [3:0]  Interrupt 
    temp = DRV_Thunder_Read(0x03) & 0x1F;    // Frequency division ratio(antenna),mask disturber, interrupt
    DRV_Thunder_Write(0x03, 0x20 | temp);
//------------------------------------------------------------------------------
    
//    DISP_LCO     [7]    Display LCO on IRQ pin
//    DISP_SRCO    [6]    Display SRCO on IRQ pin
//    DISP_TRCO    [5]    Display TRCO on IRQ pin
//    TUN_CAP      [3:0]  Internal Tuning Capacitors (from 0 to 120pF in steps of 8pf)
    DRV_Thunder_Write(0x08, 0x00);           // LCO, SRCO, TRCO on IRQ, capacitors tuning
}


/******************************************************************************
  Function:
    void DRV_THUNDER_DISABLE_SPI(void) 

  Summary:
 * de energize the i/o pins on the spi bus
    
  Description:
 The spi bus is shared and needs to be disabled prior to apply power to the Vcc
 * pin

  Parameters:
 *    none
 

  Returns:
 none
 ******************************************************************************/
void DRV_THUNDER_DISABLE_SPI(void) {

    DRV_SPI_Deinitialize(thunderInitData.channel);
}



/******************************************************************************
  Function:
    void DRV_THUNDER_POWER_UP(void) 

  Summary:
 do not have power applied to the pins of the chip prior to 
 * applying power to vcc.
 * prevent any sort of latchup.
    
  Description:
 The spi bus is shared and needs to be disabled prior to apply power to the Vcc
 * pin

  Parameters:
 *    none
 

  Returns:
 none
 ******************************************************************************/
void DRV_THUNDER_POWER_UP(void) {
    DRV_THUNDER_DISABLE_SPI();
    MIKROBUS_CS_TRIS = INPUT_PIN;   // do not power the device through the I/o
    MIKROBUS_INT_TRIS = INPUT_PIN;
    delay_ms(20); 
    THUNDERCSHigh();
    MKROE_power_on();
    delay_ms(50);                 // allow for power to stabilize
    MIKROBUS_CS_TRIS = OUTPUT_PIN;
    MIKROBUS_INT_TRIS = INPUT_PIN;
}


/******************************************************************************
  Function:
    void DRV_THUNDER_POWER_DOWN(void) 

  Summary:
 * set pins to not power the chip
    
  Description:
 When the spi bus is used to access memory the led on the module will glow.  
 * Power is flowing from the sdo into the spi input pin and leaking out into the 
 * led

  Parameters:
 *    none
 

  Returns:
 none
 ******************************************************************************/
void DRV_THUNDER_POWER_DOWN(void) {
    DRV_THUNDER_DISABLE_SPI();
    THUNDERCSLow();
    MKROE_power_off();
}


