/********************************************************************
 ********************************************************************
 * FileName:     heartrate1_hw.c
 * Dependencies:
 * Processor:	   PIC24
 * Complier:     Microchip xc16 ver 1.26
 * Company:      Electronic Design Solutions, LLC.
 *
 *********************************************************************
 * File Description:
 *
 *  This file handles the heart rate funtions
 *
 * Change History:
 *  Rev   Date         Author    Description
 *  1.0   3/9/2016    DRW       Initial revision
 ********************************************************************/
/********************************************************************
Software License Agreement:

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *******************************************************************/

/******************************************************************************
 * Includes
 *******************************************************************************/
#include "heartrate1_hal.h"
#include "heartrate1_hw.h"
#include <stdint.h>

uint16_t ir_buffer[16], red_buffer[80];
uint16_t hr_buffer[SAMPLES];
uint16_t temperature_buffer[100];

uint16_t HR_movingAVE_index;
uint16_t SPO2_movingAVE_index;
uint16_t TEMPERATURE_movingAVE_index;

int ir_screen[16], red_screen[16];
int cnt_samples, cnt;



// reset and initialize the sensor

void heartrate1_init( uint8_t slave_address ) {
    uint8_t cfg[1] = {0};
    uint8_t cfg_reset[1] = {0};

    heartrate1_i2c_init( slave_address );
    heartrate1_i2c_hal_read( MODE_CONFIG, 1, cfg_reset );

    cfg_reset[0] |= 0b01000000;

    heartrate1_i2c_hal_write( MODE_CONFIG, 1, cfg_reset );
    // wait for the RESET bit to clear itself

    while ( 1 ) {
        heartrate1_i2c_hal_read( MODE_CONFIG, 1, cfg_reset );
        if ( (cfg_reset[0]&0b01000000) == 0 ) break;
    }

    // read old and set new mode
    heartrate1_i2c_hal_read( MODE_CONFIG, 1, cfg );
    //cfg[0] = ( cfg[0] & ~0x07 ) | HR_ONLY | SPO2_EN | TEMP_EN;
    cfg[0] = ( cfg[0] & ~0x07 ) | HR_ONLY ;
    heartrate1_i2c_hal_write( MODE_CONFIG, 1, cfg );
    resetFIFO_PointerRegisters();  // this should be done every time the mode is changed

    heartrate1_i2c_hal_read( SPO2_CONFIG, 1, cfg );   // read SpO2 configuration
    cfg[0] |= ( 1 << 6 ); //SPO2_HI_RES_EN;           // enable High Resolution SpO2
    cfg[0] |= SAMPLES_100;                             // set samples per second
    cfg[0] |= PULSE_WIDTH_1600;                       // set 16-bit ADC resolution
    heartrate1_i2c_hal_write( SPO2_CONFIG, 1, cfg );  // send new SpO2 configuration

    cfg[0] = 0x00;
    cfg[0] |= IR_CURRENT_306;                       // set IR LED current to be 50 mA
    cfg[0] |= RED_CURRENT_500;                      // set RED LED current to be 24 mA
    heartrate1_i2c_hal_write( LED_CONFIG, 1, cfg ); // send new LED current control config

    // interrupt settings
    heartrate1_i2c_hal_read( INT_ENABLE, 1, cfg );
    //    cfg[0] |= ENA_HR_RDY;
    //    cfg[0] |= ENA_SO2_RDY;
    //    cfg[0] |= ENA_TEP_RDY;
    //In SpO2 and heart-rate modes, this interrupt triggers when the FIFO
    //write pointer is the same as the FIFO read pointer minus one, which
    //means that the FIFO has only one unwritten space left. If the FIFO 
    //is not read within the next conversion time, the FIFO becomes 
    //full and future data is lost.
    cfg[0] |= ENA_A_FULL;

    // send new interrupt settings
    heartrate1_i2c_hal_write( INT_ENABLE, 1, cfg );

}



void EnableTemperature(void) {
    uint8_t configRegister;

    heartrate1_i2c_hal_read( MODE_CONFIG, 1, &configRegister );
    configRegister |= 0b00001000;  //Temperature_Enable;
    heartrate1_i2c_hal_write( MODE_CONFIG, 1, &configRegister );
}
// get revision ID

uint8_t heartrate1_get_revision_id() {
    uint8_t read_data;
    heartrate1_i2c_hal_read( REVISION_ID, 1, &read_data );
    return read_data;
}

// get part ID

uint8_t heartrate1_get_part_id() {
    uint8_t read_data;
    heartrate1_i2c_hal_read( PART_ID, 1, &read_data );
    return read_data;
}

// get config

uint8_t heartrate1_get_config() {
    uint8_t read_data;
    heartrate1_i2c_hal_read( MODE_CONFIG, 1, &read_data );
    return read_data;
}

// get status

uint8_t heartrate1_get_status() {
    uint8_t read_data;
    heartrate1_i2c_hal_read( INT_STATUS, 1, &read_data );
    return read_data;
}

// send configuration

void heartrate1_set_config( uint8_t cfg ) {
    heartrate1_i2c_hal_write( MODE_CONFIG, 1, &cfg );
}

// set interrupts

void heartrate1_set_interrupt( uint8_t intrpts ) {
    heartrate1_i2c_hal_write( MODE_CONFIG, 1, &intrpts );
}

// read data from the sensor

uint8_t heartrate1_read_ir_red(uint16_t *ir_buff, uint16_t *red_buff ) {
    int8_t i;
    uint8_t sampleNum = 0xFF;

    uint8_t wrPtr = 0, rdPtr = 0, samples[4];
    uint8_t overFlowPtr;

    heartrate1_i2c_hal_read( FIFO_WRITE_PTR, 1, &wrPtr );
    heartrate1_i2c_hal_read( FIFO_READ_PTR, 1, &rdPtr );
    heartrate1_i2c_hal_read( OVER_FLOW_CNT, 1, &overFlowPtr );

    if (overFlowPtr == 0) {
        if (wrPtr > rdPtr)
            sampleNum = wrPtr - rdPtr;
        else if (rdPtr > wrPtr)
            sampleNum = 16 + (wrPtr - rdPtr);
        else
            sampleNum = 0;  //16;  // need to check overflow


        if ( sampleNum >= 1 ) {
            for ( i = 0; i < sampleNum; ++i ) {
                heartrate1_i2c_hal_read( FIFO_DATA_REG, 4, samples );
                ( ( uint8_t * ) ir_buff   )[0]  = samples[1];
                ( ( uint8_t * ) ir_buff++ )[1]  = samples[0];

                ( ( uint8_t * ) red_buff   )[0]  = samples[3];
                ( ( uint8_t * ) red_buff++ )[1]  = samples[2];
            }
        }
    } else{
        heartrate1_i2c_hal_read( FIFO_DATA_REG, 4, samples );
    }


    return sampleNum;
}




//uint8_t heartrate1_read_test(uint16_t *ir_buff, uint16_t *red_buff ) {
//    int8_t i;
//    uint8_t sampleNum = 0xFF;
//
//    uint8_t wrPtr = 0, rdPtr = 0, samples[4];
//    uint8_t overFlowPtr;
//
//  
//
//    if (overFlowPtr == 0) {
//        if (wrPtr > rdPtr)
//            sampleNum = wrPtr - rdPtr;
//        else if (rdPtr > wrPtr)
//            sampleNum = 16 + (wrPtr - rdPtr);
//        else
//            sampleNum = 16;  //16;  // need to check overflow
//
//
//        if ( sampleNum >= 1 ) {
//            for ( i = 0; i < sampleNum; ++i ) {
//                heartrate1_i2c_hal_read( FIFO_DATA_REG, 4, samples );
//                ( ( uint8_t * ) ir_buff   )[0]  = samples[1];
//                ( ( uint8_t * ) ir_buff++ )[1]  = samples[0];
//
//                ( ( uint8_t * ) red_buff   )[0]  = samples[3];
//                ( ( uint8_t * ) red_buff++ )[1]  = samples[2];
//            }
//        }
//    } else{
//        heartrate1_i2c_hal_read( FIFO_DATA_REG, 4, samples );
//    }
//
//
//    return sampleNum;
//}

// read calibration temperature from the sensor

void heartrate1_read_temp(uint16_t *temp_value ) {
    uint8_t temp_int, temp_frac;
    heartrate1_i2c_hal_read( TEMP_INTEGER, 1, &temp_int );   // read temperature - int part
    heartrate1_i2c_hal_read( TEMP_FRACTION, 1,
            &temp_frac ); // read temperature - frac part

    ( ( uint8_t * ) temp_value )[1]  = temp_int;
    ( ( uint8_t * ) temp_value )[0]  = temp_frac;
}

//void getBuffer_data(void) {
//    uint8_t i = 0;
//    uint8_t sample_num;
//    unsigned long ir_average, red_average;
//    uint16_t  ir_buff[16]  = {0}, red_buff[16] = {0};
//    uint8_t interupt_status;
//    uint16_t temperatureValue;
//
//    if (DataIsReady()) {
//        interupt_status = heartrate1_get_status();
//        if (interupt_status & 0b10110000 ) {
//            sample_num = heartrate1_read_ir_red( ir_buff, red_buff );
//            // Average data
//            if ( sample_num >= 1 ) {
//                ir_average = 0;
//                red_average = 0;
//                for ( i = 0; i < sample_num; i++ ) {
//                    ir_average += ir_buff[i];
//                    red_average += red_buff[i];
//                }
//                ir_average  /= sample_num;
//                red_average /= sample_num;
//
//                if (interupt_status & 0b00100000 ) {
//                    hr_buffer[HR_movingAVE_index] = ir_average;
//                    if (HR_movingAVE_index++ > SAMPLES)
//                        HR_movingAVE_index = 0;
//                } else {
//                    ir_buffer[SPO2_movingAVE_index] = ir_average;
//                    red_buffer[SPO2_movingAVE_index] = red_average;
//                    if (SPO2_movingAVE_index++ > SAMPLES)
//                        SPO2_movingAVE_index = 0;
//                }
//
//            }
//        } else if (interupt_status & 0b01000000 ) {
//            heartrate1_read_temp(&temperatureValue);
//            temperature_buffer[TEMPERATURE_movingAVE_index] = temperatureValue;
//            if (TEMPERATURE_movingAVE_index++ > 99)
//                TEMPERATURE_movingAVE_index = 0;
//        } else {
//            // power on reset occured
//        }
//    }
//}


//#define FIFO_WRITE_PTR   0x02
//#define OVER_FLOW_CNT    0x03
//#define FIFO_READ_PTR    0x04

/****************************************************************************
  Function:
   void resetFIFO_PointerRegisters( void )

  Description:
    This function clears the fifo write pointer, over flow counter and fifo
 * read pointer register.  

  Precondition:
 I2C must be configured

  Parameters:
    None

  Returns:
    None

  Remarks:
 Point to the first register and write 0 3 times consectutively  this will clear
 * all three registers. 
 ***************************************************************************/
void resetFIFO_PointerRegisters(void) {
    uint8_t reset_registers[3] = {0, 0, 0};

    heartrate1_i2c_hal_write( FIFO_WRITE_PTR, 3, reset_registers ) ;

}


