/********************************************************************
 ********************************************************************
 * FileName:     heratrate1_hw.h
 * Dependencies:
 * Processor:	 PIC24
 * Complier:     Microchip xc16 ver 1.25
 * Company:      Electronic Design Solutions, LLC.
 *
 *********************************************************************
 * File Description:
 *
 *  The heartrate1 is an integrated pulse oximetry and heartrate monitor sensor
 *  solution. It combines two LEDs, a photodetector, optimized optics,
 *  and low-noise analog signal processing to detect pulse oximetry and
 *  heart-rate signals.
 *  The heartrate1 operates from 1.8V and 3.3V power supplies and can be
 *  powered down through software with negligible standby current,
 *  permitting the power supply to remain connected at all times.
 *
 * Change History:
 *  Rev   Date         Author    Description
 *  1.0   3/10/2016    DRW      Initial revision
 ********************************************************************/
/********************************************************************
Software License Agreement:

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *******************************************************************/
#ifndef HEARTRATE1_HW_H
#define	HEARTRATE1_HW_H



/******************************************************************************
* Includes
*******************************************************************************/

#include "gfx/gfx.h"
#include <stdint.h>
#include <stdbool.h>

/******************************************************************************
* Preprocessor Constants
*******************************************************************************/


/******************************************************************************
* Configuration Constants
*******************************************************************************/
#define MAX30100_I2C_ADR 0x57

// Registers addresses

#define INT_STATUS       0x00
#define INT_ENABLE       0x01
#define FIFO_WRITE_PTR   0x02
#define OVER_FLOW_CNT    0x03
#define FIFO_READ_PTR    0x04
#define FIFO_DATA_REG    0x05
#define MODE_CONFIG      0x06
#define SPO2_CONFIG      0x07
#define LED_CONFIG       0x09
#define TEMP_INTEGER     0x16
#define TEMP_FRACTION    0x17
#define REVISION_ID      0xFE
#define PART_ID          0xFF


// @section Mode configuration bits
#define TEMP_EN          0x08
#define HR_ONLY          0x02
#define SPO2_EN          0x03


// @section SpO2 configuration bits
#define SPO2_HI_RES_EN   0x40

// @sectionInterrupt enable bits
#define ENA_A_FULL       0x80
#define ENA_TEP_RDY      0x40
#define ENA_HR_RDY       0x20
#define ENA_SO2_RDY      0x10

// Interrupt status bits
#define PWR_RDY          0x01
#define Temperature_Ready_Flag  0x40
#define Temperature_Enable = 0x04

// sample rate control bits [samples per second]
#define SAMPLES_50       0x00
#define SAMPLES_100      0x04
#define SAMPLES_167      0x08
#define SAMPLES_200      0x0C
#define SAMPLES_400      0x10
#define SAMPLES_600      0x14
#define SAMPLES_800      0x18
#define SAMPLES_1000     0x1C

// LED pulse width control bits - pulse width [us]
#define PULSE_WIDTH_200  0x00 // 13-bit ADC resolution
#define PULSE_WIDTH_400  0x01 // 14-bit ADC resolution
#define PULSE_WIDTH_800  0x02 // 15-bit ADC resolution
#define PULSE_WIDTH_1600 0x03 // 16-bit ADC resolution

// LED current control bits [ma]
#define IR_CURRENT_0     0x00 // 0.0 mA
#define IR_CURRENT_44    0x01 // 4.4 mA
#define IR_CURRENT_76    0x02 // 7.6 mA
#define IR_CURRENT_110   0x03 // 11.0 mA
#define IR_CURRENT_142   0x04 // 14.2 mA
#define IR_CURRENT_174   0x05 // 17.4 mA
#define IR_CURRENT_208   0x06 // 20.8 mA
#define IR_CURRENT_240   0x07 // 24.0 mA
#define IR_CURRENT_271   0x08 // 27.1 mA
#define IR_CURRENT_306   0x09 // 30.6 mA
#define IR_CURRENT_338   0x0A // 33.8 mA
#define IR_CURRENT_370   0x0B // 37.0 mA
#define IR_CURRENT_402   0x0C // 40.2 mA
#define IR_CURRENT_436   0x0D // 43.6 mA
#define IR_CURRENT_468   0x0E // 46.8 mA
#define IR_CURRENT_500   0x0F // 50.0 mA

#define RED_CURRENT_0    0x00 // 0.0 mA
#define RED_CURRENT_44   0x10 // 4.4 mA
#define RED_CURRENT_76   0x20 // 7.6 mA
#define RED_CURRENT_110  0x30 // 11.0 mA
#define RED_CURRENT_142  0x40 // 14.2 mA
#define RED_CURRENT_174  0x50 // 17.4 mA
#define RED_CURRENT_208  0x60 // 20.8 mA
#define RED_CURRENT_240  0x70 // 24.0 mA
#define RED_CURRENT_271  0x80 // 27.1 mA
#define RED_CURRENT_306  0x90 // 30.6 mA
#define RED_CURRENT_338  0xA0 // 33.8 mA
#define RED_CURRENT_370  0xB0 // 37.0 mA
#define RED_CURRENT_402  0xC0 // 40.2 mA
#define RED_CURRENT_436  0xD0 // 43.6 mA
#define RED_CURRENT_468  0xE0 // 46.8 mA
#define RED_CURRENT_500  0xF0 // 50.0 mA

#define SAMPLES 3000
#define DataIsReady()     ( MIKROBUS_INT == 0 )
    
extern uint16_t hr_buffer[SAMPLES];
extern uint16_t red_buffer[80];
    
void ShowScreenClick_demo(void);
bool ProcessMessage_DEMO_test( WORD translatedMsg, GFX_GOL_OBJ_HEADER * pObj, GFX_GOL_MESSAGE * pMsg );
void CLICK_demo(void);
void resetFIFO_PointerRegisters(void);

/**
 * @brief This function restarts and initialize heartrate1 chip.
 */
void heartrate1_init( uint8_t slave_address );

/**
 * @brief Using this function we can get revision ID.
 * @return
 */
uint8_t heartrate1_get_revision_id( void );

/**
 * @brief Using this function we can get part ID.
 * @return
 */
uint8_t heartrate1_get_part_id( void );

/**
 * @brief Using this function we can get configuration of the chip.
 * @return
 */
uint8_t heartrate1_get_config( void );

/**
 * @brief Get status of heartrate1.
 * @return
 */
uint8_t heartrate1_get_status( void );

/**
 * @brief Configure heartrate1 chip with this function.
 * @param cfg
 */
void heartrate1_set_config ( uint8_t config ); // desired config value

/**
 * @brief Set interrupt for heartrate1.
 * @param intrpts
 */
void heartrate1_set_interrupt( uint8_t intrpts ); // desired interrupts

/**
 * @brief This function read IR and RED values.
 * @param irBuff
 * @param redBuff
 * @return
 * @retval 0 - success
 */
uint8_t heartrate1_read_ir_red( uint16_t *ir_buff, uint16_t *red_buff );

/**
 * @brief This function reads temperature.
 * @param tempValue
 */
void heartrate1_read_temp( uint16_t *temp_value );
  

void getBuffer_data(void); 
void EnableTemperature(void);
void testCLICK_HR_Status(void);
//uint32_t hr_buffer[800];

#endif	/* HEARTRATE1_HW_H */

