/********************************************************************
 ********************************************************************
 * FileName:     drv_spi_RFID.c
 * Dependencies:
 * Processor:	 PIC24
 * Complier:     Microchip xc16 ver 1.26
 * Company:      Electronic Design Solutions, LLC.
 *
 *********************************************************************
 * File Description:
 *
 *  This file is the high level app
 *
 * Change History:
 *  Rev   Date         Author    Description
 *  1.0   3/9/2016    DRW       Initial revision
 ********************************************************************/
/********************************************************************
Software License Agreement:

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *******************************************************************/

#include <stdint.h>
#include "system.h"
#include "drv_spi_RFID.h"
#include "string.h"

static DRV_SPI_INIT_DATA rfidInitData;  // spi parameters

/************************************************************************
 * Macros
 ************************************************************************/
#define SPILOCK(ch)                         DRV_SPI_Lock(ch)
#define SPIUNLOCK(ch)                       DRV_SPI_Unlock(ch)
#define SPIINITIALIZE(initData)             DRV_SPI_Initialize(initData);
#define PUTSPIBYTE(ch, byte)                {                           \
                                                DRV_SPI_Put(ch, byte);  \
                                            }

#define GETSPIBYTE(ch, data)                {                           \
                                                data = DRV_SPI_Get(ch); \
                                            }

/******************************************************************************
  Function:
    void DRV_RFID_Initialize( const SYS_MODULE_INDEX index,
                                           const SYS_MODULE_INIT * const init )

  Summary:
    Initializes hardware and data for the given instance of the rfid module

  Description:
    This routine initializes hardware for the instance of the rfid module,
    using the hardware initialization given data.  It also initializes all
    necessary internal data.

  Parameters:
    pInitData - Pointer to the data structure containing all data
                necessary to initialize the hardware. This pointer may
                be null if no data is required and static initialization
                values are to be used.

  Returns:
    None
 ******************************************************************************/
void DRV_RFID_Initialize(DRV_SPI_INIT_DATA *pInitData) {
    // initialize the SPI channel to be used
    SPIINITIALIZE( pInitData);
    memcpy(&rfidInitData, pInitData, sizeof (DRV_SPI_INIT_DATA));
}

/******************************************************************************
  Function:
   void DRV_RFID_Select_ISO_IEC_14443_A_Protocol(void)

  Summary:
   This command selects the RF communication protocol and prepares the CR95HF
for communication with a contactless tag.
    
  Description:
 this function writes to select 14443-a and 106kbs. the readcmd reads the
 * response but does nothing with the result.  The first byte of this message 
 * could be checked for transmission success. 0x80 
ISO/IEC 14443 Type A tag, 106 Kbps transmission and reception
rates, Time interval 86/90
  Parameters:
 *  none

  Returns:
 * 
 *      true  0x00
 *      false 0x83 for invalid protocol; 
 *            0x82 invalid command length
 *            0xFF response timeout after 2 seconds
 ******************************************************************************/
bool DRV_RFID_Select_ISO_IEC_14443_A_Protocol(void) {
    uint8_t rdata1[20];
    uint8_t protocol[2];
    bool returnValue = false;
    uint8_t response;

    protocol[0] = 0x02;  //02: ISO/IEC 14443-A
    protocol[1] = 0x00;  //00: 106 Kbps  transmit and receive

    DRV_RFID_WriteCommand(ProtocolSelect, protocol, 2);
    response = DRV_RFID_readCmd(rdata1, 100);
    if (response == 0)
        returnValue = true;
    return returnValue;
}

/******************************************************************************
  Function:
   void DRV_RFID_IndexMod_Gain(void)

  Summary:
 Improve RF Performance by setting the modulation index and receiver gain
    
  Description:
 Adjusting the Modulation Index and Receiver Gain parameters helps improve 
 * application behavior. The default value of these parameters (Table 24) is 
 * set by the PROTOCOLSELECT command, but they can be overwritten using the 
 * Write Register (WRREG) command (0x09). Table 22 and Table 23 list possible 
 * values for the Modulation Index and Receiver Gain parameters respectively.
 * This new configuration is valid until a new PROTOCOLSELECT or Write Register
 *  (of register ARC_B) command is executed. Register values are cleared 
 * at power off.

  Parameters:
 *  none

  Returns:
 *      none
 ******************************************************************************/
bool DRV_RFID_IndexMod_Gain(void) {
    uint8_t gain[5];
    uint8_t rdata1[20];
    bool returnValue = false;
    uint8_t response;

    gain[0] = 0x68;  //Analog Register Configuration address index
    gain[1] = 0x01;  //Flag Increment address or not after Write command
    gain[2] = 0x01;  //Index pointing to the Modulation Index and Receiver 
    //Gain values in the ARC_B register (0x01)
    gain[3] = 0x50;  //New value for Modulation Index and Receiver Gain nibbles
    //modulation index 1 10%;2 17%;3 25%;4 30%;5 36%;6 36%;D 95%
    //Receiver Gain 0 34db;1 32db;3 27db;7 20db;F 8db

    DRV_RFID_WriteCommand(WrReg, gain, 4);
    response = DRV_RFID_readCmd(rdata1, 100);
    if (response == 0)
        returnValue = true;

    return returnValue;
}

/******************************************************************************
  Function:
   void DRV_RFID_AutoFDet(void)

  Summary:
 
    
  Description:
ISO/IEC 18092 Enable AutoDetect Filter to synchronize NFC Forum Tag Type 3 with
CR95HF device

  Parameters:
 *  none

  Returns:
 *      none
 ******************************************************************************/
bool DRV_RFID_AutoFDet(void) {
    uint8_t FDet[6];
    uint8_t rdata1[20];
    bool returnValue = false;
    uint8_t response;

    FDet[0] = 0x0A;  // 
    FDet[1] = 0x01;  // 
    FDet[2] = 0x02;  // 
    FDet[3] = 0xA1;  // 

    DRV_RFID_WriteCommand(WrReg, FDet, 4);
    response = DRV_RFID_readCmd(rdata1, 100);

    if (response == 0)
        returnValue = true;

    return returnValue;
}

/****************************************************************************
  Function:
BOOL DRV_RFID_GetNFCTag(uint8_t *pData)

  Description:
 This function is polled for new tag detection.
 

  Precondition:
 

  Parameters:
 pointer to array for the tag information

  Returns:
 true if valid tag is detected and read; false otherwise

  Remarks:
 * do not power the click from external power.  need to add exit demo if 
 * fails set up
 ***************************************************************************/
BOOL DRV_RFID_GetNFCTag(uint8_t *pData) {
    uint8_t sdata[4];
    uint8_t rdata1[20];
    uint8_t  *pD;
    uint8_t j;
    uint8_t errorCode;

    pD = pData;

    sdata[0] = 0x26;   //REQA
    sdata[1] = 0x07;

    DRV_RFID_WriteCommand(SendRecv, sdata, 2);
    DRV_RFID_readCmd(rdata1, 100);

    sdata[0] = 0x93;  //Anticol
    sdata[1] = 0x20;
    sdata[2] = 0x08;
    DRV_RFID_WriteCommand(SendRecv, sdata, 3);
    errorCode = DRV_RFID_readCmd(rdata1, 100);


    if (errorCode == 0x80) {
        for ( j = 0; j < 10; j++) {
            *pD++ =  rdata1[j] ;
        }
        return true;
    } else {
        return false;
    }
}

/******************************************************************************
  Function:
    void DRV_RFID_WriteCommand(uint8_t cmd, uint8_t *pData, uint8_t nCount)

  Summary:
    Writes commands to the device.
    
  Description:
    This routine unlocks the spi port, drives the chip select low,sends 
 *  the control command byte,and number of bytes.  The loop sends out the command
 * sequence

  Parameters:
 *    cmd:  send commands to the device
 *  pData:  pointer to the command
 * nCount:  number of bytes in the command

  Returns:
    None
 ******************************************************************************/
void DRV_RFID_WriteCommand(uint8_t cmd, uint8_t *pData, uint8_t nCount) {
    uint8_t  *pD;
    uint8_t counter;
    uint8_t testData;

    pD = pData;

    while (!SPILOCK(rfidInitData.channel));

    DRV_SPI_Initialize( (DRV_SPI_INIT_DATA *) & rfidInitData);
    delay_ms(20);
    RFIDCSLow();
    PUTSPIBYTE(rfidInitData.channel, SEND_CONTROL_COMMAND);
    PUTSPIBYTE(rfidInitData.channel, cmd);
    PUTSPIBYTE(rfidInitData.channel, nCount);
    for (counter = 0; counter < nCount; counter++) {
        testData = *pD;
        PUTSPIBYTE(rfidInitData.channel, *pD++);
    }
    RFIDCSHigh();
    SPIUNLOCK(rfidInitData.channel);
}

/******************************************************************************
  Function:
    BOOL DRV_RFID_EchoResponse(void) 

  Summary:
    The ECHO command verifies the possibility of communication between a Host
 and the CR95HF
    
  Description:
    This routine sends out an ECHO command and will return true if the echo
 * command is returned. After issuing the echo command the device is polled for data
 * ready flag is set.  Then the data is read and compared to ECHO.
 * need to add timeout function to handle error gracefully otherwise WDTO will occur

  Parameters:
    none.

  Returns:
 true if connection is established between pic and cr95hf, false otherwise.
 *  
 ******************************************************************************/
BOOL DRV_RFID_EchoResponse(void) {
    BOOL result = false;
    uint8_t readData;
    uint8_t tryCount;
    tryCount = 0;
    while (!SPILOCK(rfidInitData.channel));

    DRV_SPI_Initialize( (DRV_SPI_INIT_DATA *) & rfidInitData);
    delay_ms(20);
    RFIDCSLow();
    PUTSPIBYTE(rfidInitData.channel, SEND_CONTROL_COMMAND);
    PUTSPIBYTE(rfidInitData.channel, ECHO);
    RFIDCSHigh();

    while (1) {

        if (MIKROBUS_INT == 0) {
            RFIDCSLow();
            PUTSPIBYTE(rfidInitData.channel, SEND_CONTROL_READ);
            GETSPIBYTE(rfidInitData.channel, readData);
            RFIDCSHigh();
            if (readData == ECHO) {
                result = true;
            }
            break;
        }
        if (tryCount++ > 100 )
            break;
        delay_ms(20);
    }
    SPIUNLOCK(rfidInitData.channel);
    return result;
}

/******************************************************************************
  Function:
    BOOL DRV_RFID_READ_WAKEUP_REGISTER(void) 

  Summary:
    The ECHO command verifies the possibility of communication between a Host
 and the CR95HF
    
  Description:
    This routine sends out an ECHO command and will return true if the echo
 * command is returned. After issuing the echo command the device is polled for data
 * ready flag is set.  Then the data is read and compared to ECHO.
 * need to add timeout function to handle error gracefully otherwise WDTO will occur

  Parameters:
    none.

  Returns:
 true if connection is established between pic and cr95hf, false otherwise.
 *  
 ******************************************************************************/
uint8_t DRV_RFID_READ_WAKEUP_REGISTER(void) {
    uint8_t sdata[4];
    uint8_t rdata1[4];


    uint8_t response;
    uint8_t result = 0xFF;

    sdata[0] = 0x62;  //wake up event register
    sdata[1] = 0x01;
    sdata[2] = 0x00;
    DRV_RFID_WriteCommand(RdReg, sdata, 3);
    delay_ms(20);
    response = DRV_RFID_readCmd(rdata1, 2);

    if (response == 0x00)
        result = rdata1[0];

    return result;
}

/****************************************************************************
  Function:
  BOOL DRV_RFID_ID(uint8_t *pData) 

  Description:
 This function Requests short information about the CR95HF and its revision.
 

  Precondition:
 

  Parameters:
 pointer to array to hold the ID value

  Returns:
 true if valid id found; false otherwise

  Remarks:
// NFC FS2JAST4     Revision 4
// element 13 is null 
// element 12 is the revision
// elements 0-11 device string
 ***************************************************************************/
BOOL DRV_RFID_ID(uint8_t *pData) {
    uint8_t id[5];
    uint8_t rdata1[20];
    uint8_t  *pD;
    uint8_t errorCode;
    uint8_t j;

    pD = pData;
    DRV_RFID_WriteCommand(IDN, id, 0x00);
    errorCode = DRV_RFID_readCmd(rdata1, 100);

    if (errorCode == 0x00) {
        for ( j = 0; j < 13; j++) {
            *pD++ =  rdata1[j] ;
        }
        return true;
    } else {
        return false;
    }
}

/****************************************************************************
  Function:
   void DRV_IRQ_LOW_PULSE( void )

  Description:
 This function is used to select the serial protocol on system boot up and to
 * wake the device from WFE mode
 

  Precondition:
 

  Parameters:
    None

  Returns:
    None

  Remarks:
 *
 ***************************************************************************/
void DRV_IRQ_LOW_PULSE(void) {
    delay_ms(10);
    IRQ_low();                          // put IRQ_IN pin at low level
    delay_ms(20);                       // this sequence will wake the chip
    IRQ_high();                         // put IRQ_IN pin at high level
    delay_ms(10);
}

/******************************************************************************
  Function:
    void DRV_RFID_readCmd(uint8_t *pData, uint8_t)  

  Summary:
 This command is always ran after any write command.  The first byte is
 * the error code and the rest is the data packet
    
  Description:
// pData format {[error code][data1],[data2],.....[data dataNum]}    

  Parameters:
    pointer to data array.

  Returns:
 * response value, or 0xFF for timeout condition
 *  
 ******************************************************************************/
uint8_t DRV_RFID_readCmd(uint8_t *pData, uint8_t timeout) {
    uint8_t i = 0;
    uint8_t dataNum;
    uint8_t errorCode = 0xFF;
    uint8_t  *pD;
    uint8_t  timeout_counter = 0;

    pD = pData;

    while (!SPILOCK(rfidInitData.channel));
    DRV_SPI_Initialize( (DRV_SPI_INIT_DATA *) & rfidInitData);
    while (1) {
        if (MIKROBUS_INT == 0) {
            RFIDCSLow();
            PUTSPIBYTE(rfidInitData.channel, SEND_CONTROL_READ);
            GETSPIBYTE(rfidInitData.channel, errorCode);
            GETSPIBYTE(rfidInitData.channel, dataNum);
            for (i = 0; i < dataNum; i++)
                GETSPIBYTE(rfidInitData.channel, *pD++);
            break;
        }
        if (timeout_counter++ > timeout)
            break;
        delay_ms(20);
    }
    RFIDCSHigh();
    SPIUNLOCK(rfidInitData.channel);
    return errorCode;
}

/******************************************************************************
  Function:
    void DRV_RFID_Calibration(void) 

  Summary:
The calibration process consists in executing a tag detection sequence using a well-known
configuration, with no tag within the antenna RF operating volume, to determine a specific
reference value (DacDataRef) that will be reused by the host to define the tag detection
parameters (DacDataL and DacDataH).
During the calibration process, DacDataL is forced to 0x00 and the software successively
varies the DacDataH value from its maximum value (0xFE) to it minimum value (0x00). At
the end of the calibration process, DacDataRef will correspond to the value of DacDataH for
which the wake-up event switches from Timeout (no tag in the RF field) to tag detected.
To avoid too much sensitivity of the tag detection process, we recommend using a guard
band. This value corresponds to 2 DAC steps (0x08).
    
  Description:
    

  Parameters:
 none

  Returns:
 none
 *  
 ******************************************************************************/
void DRV_RFID_Calibration(void) {
    uint8_t data[20];
    uint8_t i;
    uint8_t DacDataH;
    uint8_t DacDataL;
    uint8_t DacDataRef;
    uint8_t addjustmentValue;
    uint8_t wakeUpEvent;

    DacDataL = 0;

    data[0] = 0x03;  //wake-up sources  Low pulse on IRQ_IN and Tag detection
    data[1] = 0xA1;  //wfe mode  Tag Detector Calibration
    data[2] = 0x00;  //wfe mode
    data[3] = 0xF8;  //wuctrl             tag detector calibration
    data[4] = 0x01;  //wuctrl
    data[5] = 0x18;  //leave control      Hibernate  Tag Detection
    data[6] = 0x00;  //leave control
    data[7] = 0x20;  //wu period  coefficient used to adjust the time allowed between two tag detections.
    data[8] = 0x60;  //osc start delay for HFO stabilization
    data[9] = 0x60;  //delay for DAC stabilization
    data[10] = 0x00; //define the lower and higher comparator values, respectively. 
    data[11] = 0xFC; //These values are determined by a calibration process
    data[12] = 0x3F; //number of HF swings allowed during Tag Detection
    data[13] = 0x01; //maximum number of tag detection trials

    DRV_RFID_WriteCommand(IDLE, data, 0x0E);
    DRV_RFID_readCmd(data, 100);


    DacDataRef       = 0b11111100;  //0xfc
    addjustmentValue = 0b10000000;
    for (i = 0; i < 5; i++) {
        wakeUpEvent = 0xFF;
        while (wakeUpEvent == 0xFF) {
            wakeUpEvent = DRV_RFID_READ_WAKEUP_REGISTER();
        }

        if (wakeUpEvent == 0x01)
            DacDataRef -= addjustmentValue;
        else
            DacDataRef += addjustmentValue;

        addjustmentValue >>= 1;
        data[11] = DacDataRef;
        DRV_RFID_WriteCommand(IDLE, data, 0x0E);
        DRV_RFID_readCmd(data, 100);
    }
    wakeUpEvent = 0xFF;
    while (wakeUpEvent == 0xFF) {
        wakeUpEvent = DRV_RFID_READ_WAKEUP_REGISTER();
    }
    if (wakeUpEvent == 0x01)
        DacDataRef -= 4;

    DacDataH = DacDataRef + 8;
    if (DacDataRef > 8)
        DacDataL = DacDataRef - 8;

    data[11] = DacDataH;
    data[10] = DacDataL;
    DRV_RFID_WriteCommand(IDLE, data, 0x0E);
    DRV_RFID_readCmd(data, 100);
}

/******************************************************************************
  Function:
    void DRV_RFID_WakeUP_by_TagDetection(void) 

  Summary:
The calibration process consists in executing a tag detection sequence using a well-known
configuration, with no tag within the antenna RF operating volume, to determine a specific
reference value (DacDataRef) that will be reused by the host to define the tag detection
parameters (DacDataL and DacDataH).
During the calibration process, DacDataL is forced to 0x00 and the software successively
varies the DacDataH value from its maximum value (0xFE) to it minimum value (0x00). At
the end of the calibration process, DacDataRef will correspond to the value of DacDataH for
which the wake-up event switches from Timeout (no tag in the RF field) to tag detected.
To avoid too much sensitivity of the tag detection process, we recommend using a guard
band. This value corresponds to 2 DAC steps (0x08).
    
  Description:
    

  Parameters:
 none

  Returns:
 none
 *  
 ******************************************************************************/
bool DRV_RFID_WakeUP_by_TagDetection(void) {
    uint8_t data[20];
    bool returnValue = true;

    data[0] = 0x02;  //wake-up sources  Tag detection
    data[1] = 0x21;  //wfe mode  T
    data[2] = 0x00;  //wfe mode
    data[3] = 0x79;  //wuctrl             tag detector calibration
    data[4] = 0x01;  //wuctrl
    data[5] = 0x18;  //leave control      Hibernate  Tag Detection
    data[6] = 0x00;  //leave control
    data[7] = 0x20; //0x20;  //wu period  coefficient used to adjust the time allowed between two tag detections.
    data[8] = 0x60;  //osc start delay for HFO stabilization
    data[9] = 0x60;  //delay for DAC stabilization
    data[10] = 120; //0x44;//0x54; //0x64; //define the lower and higher comparator values, respectively. 
    data[11] = 138; //0x54;//0x64; //0x74; //These values are determined by a calibration process
    data[12] = 0x3F; //number of HF swings allowed during Tag Detection
    data[13] = 0x08; //maximum number of tag detection trials

    DRV_RFID_WriteCommand(IDLE, data, 0x0E);
    if (DRV_RFID_readCmd(data, 100) == 0xFF)
        returnValue = false;
    return returnValue;

}

void DRIVER_RFID_DISABLE_SPI(void) {

    DRV_SPI_Deinitialize(rfidInitData.channel);
}


