/********************************************************************
 ********************************************************************
 * FileName:     drv_spi_RFID.h
 * Dependencies:
 * Processor:	 PIC24
 * Complier:     Microchip xc16 ver 1.26
 * Company:      Electronic Design Solutions, LLC.
 *
 *********************************************************************
 * File Description:
 *
 *  This file is the rfid driver 
 *
 * Change History:
 *  Rev   Date         Author    Description
 *  1.0   4/8/2016    DRW       Initial revision
 ********************************************************************/
/********************************************************************
Software License Agreement:

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *******************************************************************/

#ifndef _DRV_RFID_H
    #define _DRV_RFID_H




/************************************************************************
 * Section:  Includes                                                       
 ************************************************************************/
#include <stdint.h>
#include "system.h"
#include "driver/spi/drv_spi.h"



#define SSI_1_Direction             TRISBbits.TRISB2         // RB2
#define SSI_1_LAT                   LATBbits.LATB2

#define SSI_0_Direction             TRISEbits.TRISE4         // RE4
#define SSI_0_LAT                   LATEbits.LATE4

#define IRQ_IN_Direction            TRISGbits.TRISG6
#define IRQ_IN_LAT                  LATGbits.LATG6

//******** CR95HF Commands Definition ******************************************
#define    IDN                    0x01
#define    ProtocolSelect         0x02
#define    SendRecv               0x04
#define    IDLE                   0x07
#define    RdReg                  0x08
#define    WrReg                  0x09
#define    BaudRate               0x0A
#define    ECHO                   0x55

//******** Control byte is used to specify a communication type and direction ** 
#define    SEND_CONTROL_COMMAND   0x00
#define    SEND_CONTROL_POLL      0x03
#define    SEND_CONTROL_READ      0x02
#define    SEND_CONTROL_RESET     0x01

//******** Error Codes *********************************************************
#define    EFrameRecvOK           0x80
#define    EUserStop              0x85
#define    ECommError             0x86
#define    EFrameWaitTOut         0x87
#define    EInvalidSof            0x88
#define    EBufOverflow           0x89
#define    EFramingError          0x8A
#define    EEgtError              0x8B
#define    EInvalidLen            0x8C
#define    ECrcError              0x8D
#define    ERecvLost              0x8E
#define    ENoField               0x8F
#define    EUnintByte             0x90




#define IRQ_low() IRQ_IN_LAT = 0
#define IRQ_high() IRQ_IN_LAT = 1

/************************************************************************
* Macro: RFIDCSLow()                                                   
*                                                                       
* Preconditions: CS IO must be configured as output
*                                                                       
* Overview: this macro pulls down CS line                    
*                                                                       
* Input: none                                                          
*                                                                       
* Output: none                                                         
*                                                                       
************************************************************************/
    #define RFIDCSLow()   MIKROBUS_CS_LAT = 0;

/************************************************************************
* Macro: RFIDCSHigh()
*                                                                       
* Preconditions: CS IO must be configured as output
*                                                                       
* Overview: this macro set CS line to high level
*                                                                       
* Input: none                                                          
*                                                                       
* Output: none
*                                                                       
************************************************************************/
    #define RFIDCSHigh()  MIKROBUS_CS_LAT = 1

/******************************************************************************
  Function:
    void DRV_RFID_Initialize( const SYS_MODULE_INDEX index,
                                           const SYS_MODULE_INIT * const init )

  Summary:
    Initializes hardware and data for the given instance of the NVM module

  Description:
    This routine initializes hardware for the instance of the NVM module,
    using the hardware initialization given data.  It also initializes all
    necessary internal data.

  Parameters:
    pInitData - Pointer to the data structure containing all data
                necessary to initialize the hardware. This pointer may
                be null if no data is required and static initialization
                values are to be used.

  Returns:
    None
******************************************************************************/
void DRV_RFID_Initialize(DRV_SPI_INIT_DATA *pInitData);

/******************************************************************************
  Function:
    void DRV_RFID_WriteCommand(    uint8_t cmd,
                                            uint8_t *pData, 
                                            uint16_t nCount )

  Summary:
    Writes an array of bytes associated with the command.
    
  Description:
    This routine writes the array of bytes from the location pointed to by 
    pData to the given address. The number of bytes to be written is specified
    by nCount.

  Parameters:
    cmd - command 
    pData   - pointer to the source of the array
    nCount  - specifies the number of bytes to be written

  Returns:
 none
    
******************************************************************************/
void DRV_RFID_WriteCommand(uint8_t cmd, uint8_t *pData, uint8_t nCount);

/******************************************************************************
  Function:
    void DRV_RFID_readCmd( uint8_t *pData, uint8_t timeout)

  Summary:
    Reads an array of bytes after initiating a write command.
    
  Description:
    This routine reads an array of bytes in response to sending a write command. The
    read array is saved to the location pointed to by pData. The first element 
 in the data array is the error code

  Parameters:
    pData   - pointer to the destination of the read array

  Returns:
    None
******************************************************************************/
uint8_t DRV_RFID_readCmd(uint8_t *pData, uint8_t timeout);
BOOL DRV_RFID_ID(uint8_t *pData);
void DRV_IRQ_LOW_PULSE(void); 
BOOL DRV_RFID_EchoResponse(void);
void DRV_RFID_Calibration(void) ;
bool DRV_RFID_Select_ISO_IEC_14443_A_Protocol(void);
bool DRV_RFID_IndexMod_Gain(void);
bool DRV_RFID_AutoFDet(void);
BOOL DRV_RFID_GetNFCTag(uint8_t *pData);
void DRIVER_RFID_DISABLE_SPI(void);
void ShowScreenClick_demo(void);
bool ProcessMessage_DEMO_test( WORD translatedMsg, GFX_GOL_OBJ_HEADER * pObj, GFX_GOL_MESSAGE * pMsg );
void CLICK_demo(void);
void testCLICK_RFID(void);
void testCLICK_RFID_read_tag(void);
bool DRV_RFID_WakeUP_by_TagDetection(void);
uint8_t DRV_RFID_READ_WAKEUP_REGISTER(void);

#endif //_DRV_SPI_RFID_H
