/********************************************************************
 ********************************************************************
 * FileName:     rfid_main.c
 * Dependencies:
 * Processor:	 PIC24
 * Complier:     Microchip xc16 ver 1.25
 * Company:      Electronic Design Solutions, LLC.
 *
 *********************************************************************
 * File Description:
 *
 *  * Description:
     This is a sample program which demonstrates the use of the ST's
     CR95HF contactless transceiver. The CR95HF is used in the applications such 
     as near field communication (NFC) using 13.56 MHz wireless interface.
     In this example, when the RFid tag is put over the RFid Click antenna, 
     the tag ID will be displayed on the display.
 *
 *  RFid Click Board - http://www.mikroe.com/click/rfid/
 * 
 * Change History:
 *  Rev   Date         Author    Description
 *  1.0   3/13/2016    DRW       Initial revision
 ********************************************************************/
/********************************************************************
Software License Agreement:

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *******************************************************************/


#include <stdint.h>

#include "system_config.h"
#include "timer.h"
#include "drv_spi_RFID.h"

#include <string.h>
#include "stdio.h"
#include"../Utilities.h"
#include "gfx/gfx.h"


bool System_Init(void);
void RFID_POWER_UP(void);
void RFID_POWER_DOWN(void);
void Display_TAG_information(uint8_t *pData);
void errorFlash(void);

extern APP_SCREEN_UPDATE_STATES update_display;
APP_SCREEN_IR_DEMO_STATES CLICK_DEMO_STATE;
extern GFX_GOL_OBJ_SCHEME pScheme_button;
static DWORD demoTick = 0;

/****************************************************************************
  Function:
   void System_Init( void )

  Description:
 This function sets up the RFID module for operation
 

  Precondition:
 

  Parameters:
    None

  Returns:
    None

  Remarks:
 * do not power the click from external power.  need to add exit demo if 
 * fails set up
 ***************************************************************************/
bool System_Init(void) {

    RFID_POWER_UP();
    if (DRV_RFID_EchoResponse() ==  false)                    return false;
    if (DRV_RFID_IndexMod_Gain() ==  false)                   return false;
    if (DRV_RFID_AutoFDet() ==  false)                        return false;
    return true;
}

/****************************************************************************
  Function:
   void RFID_POWER_UP( void )

  Description:
    This function will properly sequence power and initialize I/O pins to prevent
 * latch up.  External power can not be used to source power to the device.  The
 * device needs to sequence power and the Int_I pin to select the spi mode of 
 * communication
 

  Precondition:
 

  Parameters:
    None

  Returns:
    None

  Remarks:
 * do not power the click from external power.  could put this function
 * in the main init power up routine
 ***************************************************************************/
void RFID_POWER_UP(void) {
    // Configure RFid Click pins
    DRIVER_RFID_DISABLE_SPI();
    MIKROBUS_CS_TRIS = INPUT_PIN;   // do not power the device through the I/o
    IRQ_IN_Direction = INPUT_PIN;
    SSI_0_Direction  = INPUT_PIN;
    SSI_1_Direction  = INPUT_PIN;
    MIKROBUS_INT_TRIS = INPUT_PIN;
    delay_ms(20);
    IRQ_high();
    RFIDCSHigh();
    SSI_1_LAT = 0;                // select spi comunicaton
    SSI_0_LAT = 1;
    MKROE_power_on();
    delay_ms(20);                 // allow for power to stabilize
    MIKROBUS_CS_TRIS = OUTPUT_PIN;
    IRQ_IN_Direction = OUTPUT_PIN;
    SSI_0_Direction  = OUTPUT_PIN;
    SSI_1_Direction  = OUTPUT_PIN;
    MIKROBUS_INT_TRIS = INPUT_PIN;

    DRV_IRQ_LOW_PULSE();          // right after power up this will
    // choose the spi com mode
}


/****************************************************************************
  Function:
   void RFID_POWER_DOWN( void )

  Description:
    This function will properly power down the rfid click module.  sinse the 
 * spi is shared with the memory chip the active sdo line will pass current
 * though the chip and supply current to the led. 
 

  Precondition:
 

  Parameters:
    None

  Returns:
    None

  Remarks:

 ***************************************************************************/
void RFID_POWER_DOWN(void) {
    DRIVER_RFID_DISABLE_SPI();
    MKROE_power_off();
    IRQ_low();
    RFIDCSLow();
    SSI_1_LAT = 0;                // select spi comunicaton
    SSI_0_LAT = 0;
}





/****************************************************************************
  Function:
   void testCLICK_RFID_read_tag( void )

  Description:
    This function will set up the rfid reader parameters and read a tag with out
 * the need to invoke the display.  The led will blink when a proper rfid tag
 * is read.  Break points can be place inside the DRV_RFID_GetNFCTag function
 * to verify the tag id.
 

  Precondition:
 

  Parameters:
    None

  Returns:
    None

  Remarks:
 there is no error checking and the routine could hang in the while loop waiting
 * for the receive data interrupt line to indicate that it has data and ready to
 * send it to the host
 ***************************************************************************/
void testCLICK_RFID_read_tag(void) {
    uint8_t TAG_ID[20];

    RCONbits.SWDTEN = 0;                  // disable watch dog timer

    if (System_Init() == false)
        errorFlash();

    DRV_RFID_Calibration();  // this does not work.  found wake up parameters manually

    while (1) {
        DRV_RFID_WakeUP_by_TagDetection();
        if (DRV_RFID_READ_WAKEUP_REGISTER() == 0x02 ) {
            DRV_RFID_Select_ISO_IEC_14443_A_Protocol();
            delay_ms(100);
            if (DRV_RFID_GetNFCTag(TAG_ID)) {       // Get tag ID
                status_1_pin = 1;
                delay_ms(100);
                status_1_pin = 0;
            }
        }
    }

}

void errorFlash(void) {
    while (1) {
        status_1_pin ^= 1;
        delay_ms(100);
    }
}

/****************************************************************************
  Function:
    void ShowScreenClick_demo( void )

  Description:
    This function erases the currently displayed screen and replaces it with
    a screen consisting of:
 * a one line title
 * 3 line of instructions

    The information for the screen contents is obtained from the global
    pageInformation structure.

  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Remarks:
    The global list of graphics objects is destroyed and replaced.  Therefore,
    this function cannot be called during the message processing of
    GOLMsgCallback().  It must be called during GOLDrawCallback() processing.
 ***************************************************************************/
void ShowScreenClick_demo(void) {
    strcpy(pageInformation.title, "RFID DEMO ");
    InitializeScreen(); // set up logo and title text and destroys link list
    // of objects

    strcpy(pageInformation.instructions2, "RFID Demo ");
    strcpy(pageInformation.instructions1, "ST CR95HF");
    strcpy(pageInformation.instructions, "");

    strcpy(pageInformation.instructions3, "");
    strcpy(pageInformation.buttonTextBottomLeft, " OK");

    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS_2,
            0, 21, 127, 21 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10) + 4,
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions2,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);

    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS_1,
            0, 34, 127, 34 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10) + 4,
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions1,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);


    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS,
            0, 47, 127, 47 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10) + 4,
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);


    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS_3,
            0, 61, 127, 61 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10) + 4,
            GFX_GOL_STATICTEXT_DRAW_STATE,
            pageInformation.instructions3,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);


    GFX_GOL_ButtonCreate(ID_BUTTON_GENERIC_HOME, // ID
            49, 81, 74, 95, 2,                // dimension and radius
            GFX_GOL_BUTTON_DRAW_STATE ,       // will be displayed after creation
            NULL,                             // bitmap
            NULL,                             // no release image used
            (GFX_XCHAR*) pageInformation.buttonTextBottomLeft, // text
            GFX_ALIGN_HCENTER | GFX_ALIGN_VCENTER,
            &pScheme_button); //
}

/****************************************************************************
  Function:
   void Display_TAG_information( void )

  Description:
    This function will properly update the display with new rfid tag id info
 

  Precondition:
 Graphic objects must be created by calling ShowScreenRFID_demo()

  Parameters:
    None

  Returns:
    None

  Remarks:
    
 ***************************************************************************/
void Display_TAG_information(uint8_t *pData) {
    uint8_t TAG[4];
    uint32_t TAG_ID;
    uint32_t divisor;
    uint32_t temp;
    uint8_t decimal[10];

    uint8_t i = 0;
    divisor = 1000000000;
    TAG_ID = 0;


    TAG[0] = *pData++;  // order precedence get data then increment pointer
    TAG[1] = *pData++;
    TAG[2] = *pData++;
    TAG[3] = *pData;

    temp = TAG[3];
    temp = temp << 24;
    TAG_ID |= temp;
    temp = TAG[2];
    temp = temp << 16;
    TAG_ID |= temp;
    temp = TAG[1];
    temp = temp << 8;
    TAG_ID |= temp;
    temp = TAG[0];
    TAG_ID |= temp;

    for (i = 0; i < 9; i++) {
        decimal[i] = TAG_ID / divisor;
        TAG_ID %= divisor;
        divisor /= 10;
    }
    decimal[9] = TAG_ID;

    sprintf((char *) &(pageInformation.instructions2),
            (char*) "TAG: %02x %02x %02x %02x",
            TAG[3], TAG[2], TAG[1], TAG[0]);
    pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_2);
    GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions2);
    GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);

    sprintf((char *) &(pageInformation.instructions1), (char*) "%1d%1d%1d%1d%1d%1d%1d%1d%1d%1d ",
            decimal[0], decimal[1], decimal[2], decimal[3], decimal[4], decimal[5], decimal[6], decimal[7], decimal[8], decimal[9]);
    pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_1);
    GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions1);
    GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);

    sprintf((char *) &(pageInformation.instructions), (char*) " ");
    pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS);
    GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions);
    GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);

    update_display = GLOBAL_UPDATE; //
}

/****************************************************************************
  Function:
   void CLICK_demo( void )

  Description:
    This function initializes the hardware.  Shows tag Id in hex format
 

  Precondition:
 Graphic objects must be created by calling ShowScreenRFID_demo()

  Parameters:
    None

  Returns:
    None

  Remarks:
    The ProcessMessage_DEMO_test() function will exit the test and return to 
 * the main display
 ***************************************************************************/
void CLICK_demo(void) {
    uint8_t TAG_ID[11];
    uint8_t deviceID[20];
    static uint8_t splashScreen;
    static uint16_t refreshRate = 1000;
    uint8_t tryCount;

    if (tick > demoTick ) {
        demoTick = tick + refreshRate;
        switch (CLICK_DEMO_STATE) {
            case INITIALIZE:
                power_mode(HIGH_POWER);
                refreshRate = 1000;  // ten seconds
                System_Init();
                splashScreen = 0;
                CLICK_DEMO_STATE = SPLASH_SCREEN;
                break;
            case SPLASH_SCREEN:
                    power_mode(HIGH_POWER);
                    DRV_RFID_ID(deviceID);
                    strcpy(pageInformation.instructions1, (char *) deviceID);
                    pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_1);
                    GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions1);
                    GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                    strcpy(pageInformation.instructions, "ST CR95HF");
                    pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS);
                    GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions);
                    GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                    update_display = GLOBAL_UPDATE;
                    CLICK_DEMO_STATE = READY_TO_SCAN;
                break;
            case READY_TO_SCAN:
                power_mode(HIGH_POWER);

                strcpy(pageInformation.instructions, "Ready to scan");
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                CLICK_DEMO_STATE = SET_UP_WAKE_UP_EVENT;
                refreshRate = 50;  //
                update_display = GLOBAL_UPDATE;
                break;
            case SET_UP_WAKE_UP_EVENT:
                power_mode(HIGH_POWER);
                DRV_RFID_WakeUP_by_TagDetection();
                CLICK_DEMO_STATE = DISPLAY_RFID;
                break;
            case DISPLAY_RFID:
                power_mode(HIGH_POWER);
                if (DRV_RFID_READ_WAKEUP_REGISTER() == 0x02 ) {
                    DRV_RFID_Select_ISO_IEC_14443_A_Protocol();

                    tryCount = 0;
                    while (tryCount++ < 10) {
                        delay_ms(10);
                        if (DRV_RFID_GetNFCTag(TAG_ID)) {       // Get tag ID
                            Display_TAG_information(TAG_ID);    // need to be in high power 
                            break;                              // for display update
                        }
                    }
                    CLICK_DEMO_STATE = SET_UP_WAKE_UP_EVENT;
                }
                break;
            case EXIT_DEMO:
                RFID_POWER_DOWN();
                screenState = SCREEN_DISPLAY_MAIN;
                power_mode(HIGH_POWER);
                strcpy(pageInformation.instructions1, "Exit Demo");
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_1);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions1);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                update_display = GLOBAL_UPDATE;
                break;
        }
    }
}

/****************************************************************************
  Function:
    bool ProcessMessage_DEMO_test( WORD translatedMsg, GFX_GOL_OBJ_HEADER* pObj,
                        GFX_GOL_MESSAGE* pMsg )

  Description:
    This function processes the messages for Click demo test selection screen. 

  Precondition:
    Call ShowScreenIR_demo() prior to using this function to display the
    correct screen and generate the required graphic objects.

  Parameters:
    WORD translatedMsg  - The translated control-level message
    GFX_GOL_OBJ_HEADER* pObj    - Object to which the message applies
    GFX_GOL_MESSAGE* pMsg       - The original system message information

  Return Values:
    0   - Do not call the default message handler.
    1   - Call the default message handler.

  Remarks:
    global change variable to used  clean up later.
 ***************************************************************************/
bool ProcessMessage_DEMO_test(WORD translatedMsg, GFX_GOL_OBJ_HEADER* pObj, GFX_GOL_MESSAGE * pMsg) {
    WORD controlID;
    controlID = GFX_GOL_ObjectIDGet(pObj);
    switch (controlID) {
        case ID_BUTTON_GENERIC_HOME:
            if (translatedMsg == GFX_GOL_BUTTON_ACTION_PRESSED) {
                CLICK_DEMO_STATE = EXIT_DEMO;
                return false; // Hidden button

            } else {
                return false;
            }
            break;
        default:
            return false;
    }

    return true; // Call the default handler to show the button state.
}


