/********************************************************************
 ********************************************************************
 * FileName:     Network.c
 * Dependencies:
 * Processor:	 PIC24
 * Complier:     Microchip xc16 ver 1.25
 * Company:      Electronic Design Solutions, LLC.
 *
 *********************************************************************
 * File Description:
 *
 *  This file runs the Stop Watch Demo
 *
 * Change History:
 *  Rev   Date         Author    Description
 *  1.0   9/23/2013    DRW       Initial revision
 ********************************************************************/


#include <stdlib.h>
#include "system.h"
#include "miwi_config.h"

#include "Network.h"
#include "Utilities.h"
#include"../framework/miwi/miwi_api.h"
#include "resources/font_resources_epd_demo.h"
#include "revision_history.h"
#include "Battery_utility.h"

#include <string.h>
#include <stdio.h>


extern ACTIVE_SCAN_RESULT ActiveScanResults[ACTIVE_SCAN_RESULT_SIZE];
extern BOOL NetFreezerEnable;
extern DIAGNOSTIC_STATES testState;
extern GFX_GOL_OBJ_SCHEME pScheme_button;
extern APP_SCREEN_UPDATE_STATES update_display;
extern GFX_GOL_OBJ_SCHEME VB10_Scheme;
extern BYTE UID48[6];

BYTE myChannel = my_channel;   // Possible channel numbers are from 0 to 31

unsigned char updateDisplay;

DWORD joinNetworkDelayMs = 0;
BOOL Network = FALSE;
volatile BYTE PeerIndex;




#define ICD 8
#define none 5

BOOL radioInitialized = FALSE;

#define MAX_ATTEMPT  20


/****************************************************************************
  Function:
   void sendPowerDownMessage( void )

  Description:
    This function broadcasts a power down message.  It is repeated 3 times
    to help ensure remote devices are powered down.  There is no ACK from remotes
    on a broadcast message

  Precondition:
    radio initialized

  Parameters:
    None

  Returns:
    None

  Remarks:
    None
 ***************************************************************************/
void sendPowerDownMessage(void) {
        BYTE transmittCount;
        for (transmittCount = 0; transmittCount < 1; transmittCount++) {
            MiApp_FlushTx();
            MiApp_WriteData(POWER_DOWN); 
            MiApp_BroadcastPacket(NO_ENCRYPTION);
            delay_ms(300); // no ACK, send out again to make sure broadcast was heard
        }
}


/////////////////////////////// INITIALIZE RADIO ///////////////////////////////

/****************************************************************************
  Function:
    void ShowScreenInitializeRadio( void )

  Description:
    This function erases the currently displayed screen and replaces it with
    a screen consisting of:
 * the Microchip icon
 * a one line title
 * 3 line of instructions

    The information for the screen contents is obtained from the global
    pageInformation structure.

  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Remarks:
    The global list of graphics objects is destroyed and replaced.  Therefore,
    this function cannot be called during the message processing of
    GOLMsgCallback().  It must be called during GOLDrawCallback() processing.
 ***************************************************************************/
void ShowScreenInitializeRadio(void) {
    strcpy(pageInformation.title, "Initialize");
    InitializeScreen(); // set up logo and title text and destroys link list
    // of objects


    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS_2,
            0, 21, 127, 21 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10),
            GFX_GOL_STATICTEXT_DRAW_STATE,
             pageInformation.instructions2,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);

    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS_1,
            0, 34, 127, 34 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10),
            GFX_GOL_STATICTEXT_DRAW_STATE,
             pageInformation.instructions1,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);


    GFX_GOL_StaticTextCreate(ID_INSTRUCTIONS,
            0, 47, 127, 47 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10),
            GFX_GOL_STATICTEXT_DRAW_STATE,
             pageInformation.instructions,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);

}

/****************************************************************************
  Function:
   void initializeRadio( void )

  Description:
    This function initializes the radio when powered on.  Shows hardware version
 software version, company name, battery condition, unit id, pan id, channel

  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Remarks:
    None
 ***************************************************************************/
void initializeRadio(void) {
    volatile BYTE scanresult = 0;
    static BYTE scanCount = 1;
    static int elapsedSeconds = 0;
    static unsigned int coulombCNT;
    
    if (tick > tick_10ms + 100) {
        switch (elapsedSeconds) {
            case 0:
                power_mode(HIGH_POWER); 
                ShowScreenInitializeRadio(); // create text objects
                break;
            case 2:
                power_mode(HIGH_POWER);
                strcpy(pageInformation.instructions2, VERSION_STRING);
                strncat(pageInformation.instructions2, HARDWARE_VERSION_STRING, 2);
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_2);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions2);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                strcpy(pageInformation.instructions1, " Electronic Design ");
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_1);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions1);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                strcpy(pageInformation.instructions, "Solutions");
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                update_display = GLOBAL_UPDATE;
                break;
            case 4:
                coulombCNT = read_gasGauge();
                break;
            case 6:
                power_mode(HIGH_POWER);

#ifdef PAN_COORDINATOR
                strcpy(pageInformation.instructions1, "PAN Coordinator");
#else
                strcpy(pageInformation.instructions1, " End Device ");
#endif
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_1);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions1);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);

                if (coulombCNT > 40000) {
                    strcpy(pageInformation.instructions, "Battery Status: Good");
                } else if(coulombCNT < 20000) {
                    strcpy(pageInformation.instructions, "Battery Status: Bad");
                } else
                    strcpy(pageInformation.instructions, "Battery Status: OK");

                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                update_display = GLOBAL_UPDATE;
                break;
            case 8:
                power_mode(HIGH_POWER);
                strcpy(pageInformation.instructions2, "OSC: ");
                if (OSCCONbits.COSC == 3)
                    strcat(pageInformation.instructions2, "XTPLL 32Mhz");
                else if (OSCCONbits.COSC == 1)
                    strcat(pageInformation.instructions2, "FRCPLL 32Mhz ");
                else if (OSCCONbits.COSC == 0)
                    strcat(pageInformation.instructions2, "FRC 8Mhz");
                else if (OSCCONbits.COSC == 2)
                    strcat(pageInformation.instructions2, "XT 8Mhz");
                else
                    strcat(pageInformation.instructions2, " ??? ");
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_2);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions2);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                update_display = GLOBAL_UPDATE;
                break;
            case 10: 
                power_mode(HIGH_POWER);
                strcpy(pageInformation.instructions, "init Radio");
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                MiApp_ProtocolInit(FALSE); 
                if (MiApp_SetChannel(myChannel) == FALSE) {
                    strcpy(pageInformation.instructions1, " Unable to set channel ");
                } else {
                    sprintf((char *) &(pageInformation.instructions1), (char*) "using channel  %2d ", myChannel);
                }
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_1);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions1);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                update_display = GLOBAL_UPDATE;
                break;
          
            case 14:
                power_mode(HIGH_POWER);
                sprintf((char *) &(pageInformation.instructions1), 
                    (char*) "ID:%02x %02x %02x %02x %02x %02x",  
                    UID48[0], UID48[1], UID48[2], UID48[3], UID48[4], UID48[5]);
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_1);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions1);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                sprintf((char *) &(pageInformation.instructions), (char*) "PAN ID  %4x ", MY_PAN_ID);
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                update_display = GLOBAL_UPDATE;
                break;

            case 16:
                power_mode(HIGH_POWER);
               // MiMAC_SetPower(TX_POWER_0dB);     
                scanCount = 1;
                strcpy(pageInformation.instructions2, "  ");
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_2);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions2);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                strcpy(pageInformation.instructions1, " ");
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_1);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions1);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                 
#ifdef PAN_COORDINATOR
                strcpy(pageInformation.instructions, "Pan Coordinator");
#else
                
                sprintf((char *) &(pageInformation.instructions), (char*) "End Device   %2d ", UID48[5]); 
                
#endif
                pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS);
                GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions);
                GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                update_display = GLOBAL_UPDATE;
                break;
                
            case 18:
                power_mode(HIGH_POWER);
#ifdef PAN_COORDINATOR
                /*******************************************************************/
                //  Set the connection mode. The possible connection modes are:
                //      ENABLE_ALL_CONN:    Enable all kinds of connection
                //      ENABLE_PREV_CONN:   Only allow connection already exists in
                //                          connection table
                //      ENABL_ACTIVE_SCAN_RSP:  Allow response to Active scan
                //      DISABLE_ALL_CONN:   Disable all connections.
                /*******************************************************************/
                MiApp_ConnectionMode(ENABLE_ALL_CONN);
                
                /*******************************************************************/
                // Create a New Network
                /*******************************************************************/
                
                //Start PAN directly without noise scan  TODO  allow for search for best channel
                if (MiApp_StartConnection(START_CONN_DIRECT, 0, 0)) {
                    strcpy(pageInformation.instructions2, "Created Network");
                    pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_2);
                    GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions2);
                    GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                    strcpy(pageInformation.instructions1, "Successfully");
                    pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_1);
                    GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions1);
                    GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                    radioInitialized = TRUE;
                } else {
                    strcpy(pageInformation.instructions2, "Network Failed");
                    pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_2);
                    GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions2);
                    GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                }

               // "Address: %02x%02x", myShortAddress.v[1], myShortAddress.v[0]);

#else


                /*******************************************************************/
                // Perform an active scan
                /*******************************************************************/
                radioInitialized = TRUE;
                if (myChannel < 8)
                    scanresult = MiApp_SearchConnection(10, (0x00000001 << myChannel));
                else if (myChannel < 16)
                    scanresult = MiApp_SearchConnection(10, (0x00000100 << (myChannel - 8)));
                else if (myChannel < 24)
                    scanresult = MiApp_SearchConnection(10, (0x00010000 << (myChannel - 16)));
                else
                    scanresult = MiApp_SearchConnection(10, (0x01000000 << (myChannel - 24)));

                if (scanresult > 0) {

                    if ((ActiveScanResults[0].PANID.v[1] == myPANID.v[1]) &
                            (ActiveScanResults[0].PANID.v[0] == myPANID.v[0])) {
                        /*******************************************************************/
                        // Establish Connection and Display results of connection
                        /*******************************************************************/
                        PeerIndex = MiApp_EstablishConnection(0, CONN_MODE_DIRECT);
                        if (PeerIndex == 0xFF) {
                            strcpy(pageInformation.instructions2, "Join Failed!!! ");
                            pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_2);
                            GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions2);
                            GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                            elapsedSeconds = 10; //stay here until found network
                        } else {
                            Network = TRUE;

                            //strcpy(pageInformation.instructions2, "Join Network Success ");
                            sprintf((char *) &(pageInformation.instructions2), (char*) "Peer Index  %4x ", PeerIndex);
                            pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_2);
                            GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions2);
                            GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);

                        }
                    }
                } else {
                    sprintf((char *) &(pageInformation.instructions2), (char*) "Retry Cnt  %2d ", scanCount);
                    pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_2);
                    GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject,  pageInformation.instructions2);
                    GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
                    if (scanCount++ < 10) // give up after 10
                        elapsedSeconds = 16; 
                }
#endif
               update_display = GLOBAL_UPDATE;
                break;

            case 20:
                power_mode(HIGH_POWER);
                screenState = SCREEN_DISPLAY_MAIN;
                elapsedSeconds = 0;
                return;
                break;
        }
        tick_10ms = tick;
        elapsedSeconds++;
    }
}

/****************************************************************************
  Function:
   void joinNetwork( void )

  Description:
    This function will attempt to join the network once every second.

  Precondition:
    Radio must be initialized

  Parameters:
    None

  Returns:
    None

  Remarks:
    After joining the network the Network flag is set to TRUE
 ***************************************************************************/
void joinNetwork(void) {
    volatile BYTE scanresult = 0;
    if (Network == FALSE)
        if (tick > joinNetworkDelayMs + 3000) {
            joinNetworkDelayMs = tick;
            if (radioInitialized) {
                if (myChannel < 8)
                    scanresult = MiApp_SearchConnection(5, (0x00000001 << myChannel));
                else if (myChannel < 16)
                    scanresult = MiApp_SearchConnection(5, (0x00000100 << (myChannel - 8)));
                else if (myChannel < 24)
                    scanresult = MiApp_SearchConnection(5, (0x00010000 << (myChannel - 16)));
                else
                    scanresult = MiApp_SearchConnection(5, (0x01000000 << (myChannel - 24)));

                if (scanresult > 0) {

                    if ((ActiveScanResults[0].PANID.v[1] == myPANID.v[1]) &
                            (ActiveScanResults[0].PANID.v[0] == myPANID.v[0])) {
                        /*******************************************************************/
                        // Establish Connection
                        /*******************************************************************/
                        PeerIndex = MiApp_EstablishConnection(0, CONN_MODE_DIRECT);
                        if (PeerIndex != 0xFF) {
                            Network = TRUE;
                        }
                    }
                }
            }
        }
}


