/**
* \brief The interface for external application wants to update EPD
*
* Copyright (c) 2013-2015 Pervasive Displays Inc. All rights reserved.
*
*  Authors: Pervasive Displays Inc.
*
*  Redistribution and use in source and binary forms, with or without
*  modification, are permitted provided that the following conditions
*  are met:
*
*  1. Redistributions of source code must retain the above copyright
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in
*     the documentation and/or other materials provided with the
*     distribution.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include  "EPD_interface.h"
#include "EPD_hardware_driver.h"
#include "drv_gfx_pdi_e_paper.h"
static bool EPD_PowerStatusEnabled = false;

/**
 * \brief Initialize the EPD hardware setting 
 */
void EPD_display_init(void)
{
    DRV_EPD_DISPLAY_RESET_LAT = 0;
    DRV_EPD_DischargeReset();
    DRV_EPD_SPI_CS_LAT     = 0;               // active low but should set to 0
    DRV_EPD_BorderDisable();            
    
    DRV_EPD_DisplayDriverInitialize();
	DRV_EPD_SPI_Enable();               
}


/**
 * \brief Show image from memory by global update
 *
 * \param EPD_type_index The defined EPD size
 * \param previous_image_address The address of memory that stores previous image
 * \param new_image_address The address of memory that stores new image
 * \param On_EPD_read_memory External function to read memory
 */
void EPD_display_global(    uint8_t EPD_type_index,
                            EInt previous_image_address,
                            EInt new_image_address,
                            EPD_read_memory_handler On_EPD_read_memory)
{   
    /* Initialize EPD hardware */

    /* Power on COG Driver */
    EPD_power_on(EPD_type_index,DRV_EPD_TemperatureGet());  //this function uses the value from the previous conversion
    start_a_d_conversion();  

    /* Initialize COG Driver */
    EPD_initialize_driver();

    /* Display image data on EPD from SRAM memory */
    EPD_display_from_flash_prt(    previous_image_address,
                                    new_image_address,
                                    On_EPD_read_memory);

    end_a_d_conversion();  
    /* Power off COG Driver */
    EPD_power_off ();

    /* set power status */
    EPD_PowerStatusEnabled = false;

}
void EPD_display_global_RAM( uint8_t EPD_type_index,
                             uint8_t *previous_image_address,
                             uint8_t *new_image_address)
{
     DRV_EPD_PowerUp();
    /*
    // Power on COG Driver
    EPD_power_on(EPD_type_index,DRV_EPD_TemperatureGet());

    // Initialize COG Driver
    EPD_initialize_driver();
    */
    /* Display image data on EPD from SRAM memory */
    EPD_display_from_array_prt(previous_image_address,new_image_address);
    /* Power off COG Driver */
   // EPD_power_off ();
        DRV_EPD_PowerDown();
}
/**
 * \brief Show image from memory by partial update
 *
 * \note
 * - The EPD_init, power on and initialize COG just run at the beginning of partial
 *   update cycle, not per pattern.
 * - The difference of Partial Update from Global Update is if there is next
 *   pattern to be updated on EPD, user is able to write next image data to
 *   memory and COG without power off the COG for better visual experience and
 *   faster update time.
 * - The timing to use EPD_power_off for partial update is by use case and ghosting
 *   improvement.
 *
 * \param EPD_type_index The defined EPD size
 * \param previous_image_address The address of memory that stores previous image
 * \param new_image_address The address of memory that stores new image
 * \param On_EPD_read_memory External function to read memory
 */
void EPD_display_partial(   uint8_t EPD_type_index,
                            EInt previous_image_address,
                            EInt new_image_address,
                            EPD_read_memory_handler On_EPD_read_memory)
{
    
#ifdef G2_Aurora_Mb
    EPD_display_partialupdate_from_flash_prt(  previous_image_address,
                                    new_image_address,
                                    On_EPD_read_memory);
#endif
}
void EPD_display_partial_RAM(uint8_t EPD_type_index,
                            uint8_t *previous_image_address,
                            uint8_t *new_image_address)
{
    #ifdef G2_Aurora_Mb
    EPD_display_partialupdate_from_array_prt(previous_image_address,
                                             new_image_address);
    #endif
}
/**
 * \brief Initialize the EPD hardware setting and COG driver
 *
 * \param EPD_type_index The defined EPD size 
 */
void EPD_power_init(uint8_t EPD_type_index)
{

    /* Power on COG Driver */
    EPD_power_on(EPD_type_index,DRV_EPD_TemperatureGet());

    /* Initialize COG Driver */
    EPD_initialize_driver();

    /* set power status */
    EPD_PowerStatusEnabled = true;
}

/**
 * Power off COG Driver
 */
void EPD_power_end(void)
{
    EPD_power_off();
    
    /* set power status */
    EPD_PowerStatusEnabled = false;
}

bool EPD_PowerStatusGet(void)
{
    return EPD_PowerStatusEnabled;
}
