/********************************************************************
 Software License Agreement:

 The software supplied herewith by Microchip Technology Incorporated
 (the "Company") for its PIC(R) Microcontroller is intended and
 supplied to you, the Company's customer, for use solely and
 exclusively on Microchip PIC Microcontroller products. The
 software is owned by the Company and/or its supplier, and is
 protected under applicable copyright laws. All rights are reserved.
 Any use in violation of the foregoing restrictions may subject the
 user to criminal sanctions under applicable laws, as well as to
 civil liability for the breach of the terms and conditions of this
 license.

 THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *******************************************************************/

/** INCLUDES *******************************************************/


#include <stdint.h>
#include <stdbool.h>
#include <stdio.h>
#include <ctype.h>

#include "system.h"
#include "usb.h"
#include "usb_host_msd.h"
#include "fileio.h"
#include "app_host_msd_data_logger.h"
#include "rtcc.h"
#include "Utilities.h"
#include "stripChart.h"




int fileHeader(char, char);
int addTimeStamp(void);
int addLineTermination(void);
int addCommaSeparator(void);

const char mediaRequest[] = "Insert USB drive...";
static FILEIO_OBJECT myFile;

static char printBuffer[64];  // 


static USB_HOST_MSD_DATA_LOGGER_DEMO_STATE USBdemoState;
static uint8_t deviceAddress = 0;
static volatile bool sampleRequested = false;
char Capturename[13];
bool allowStateChange;          // control changes state to ATTACHED
extern APP_SCREEN_UPDATE_STATES update_display;
extern GFX_GOL_OBJ_SCHEME  pScheme_button;

// Declare a FILEIO_DRIVE_CONFIG structure to describe which functions 
// the File I/O library will use to communicate with the media
const FILEIO_DRIVE_CONFIG gUSBDrive = {
    (FILEIO_DRIVER_IOInitialize) NULL, // Function to initialize the I/O pins used by the driver.
    (FILEIO_DRIVER_MediaDetect) USBHostMSDSCSIMediaDetect, // Function to detect that the media is inserted.
    (FILEIO_DRIVER_MediaInitialize) USBHostMSDSCSIMediaInitialize, // Function to initialize the media.
    (FILEIO_DRIVER_MediaDeinitialize) USBHostMSDSCSIMediaDeinitialize, // Function to de-initialize the media.
    (FILEIO_DRIVER_SectorRead) USBHostMSDSCSISectorRead, // Function to read a sector from the media.
    (FILEIO_DRIVER_SectorWrite) USBHostMSDSCSISectorWrite, // Function to write a sector to the media.
    (FILEIO_DRIVER_WriteProtectStateGet) USBHostMSDSCSIWriteProtectState, // Function to determine if the media is write-protected.
};

/*********************************************************************
 * Function: void APP_HostMSDDataLoggerInitialize(void);
 *
 * Overview: Initializes the demo code
 *
 * PreCondition: None
 *
 * Input: None
 *
 * Output: None
 *
 ********************************************************************/
void APP_HostMSDDataLoggerInitialize() {
    USBdemoState = WAITING_FOR_ATTACH;
}

/*********************************************************************
 * Function: void APP_MountDrive(uint8_t address)
 *
 * Overview: This is called from the USB_ApplicationEventHandler() which is
 *   interrupt driven when the usb drive is plugged in.  this advances to the 
 *   next state
 *
 * PreCondition: apply 5 volts to usb power rail
 *
 * Input: usb device address
 *
 * Output: None
 *
 ********************************************************************/
void APP_MountDrive(uint8_t address) {
    if (USBdemoState == WAITING_FOR_ATTACH) {
        deviceAddress = address;
        USBdemoState = NON_BLOCK_DELAY_MOUNT_DRIVE;
    }
}

/*********************************************************************
 * Function: void APP_ReleasePower(void)
 *
 * Overview: This is called from the USB_ApplicationEventHandler() which is
 *   interrupt driven when the usb drive is removed.  this advances to the 
 *   next state and powers down the 5 volt usb rail
 *
 * PreCondition: apply 5 volts to usb power rail
 *
 * Input: None
 *
 * Output: None
 *
 ********************************************************************/
void APP_ReleasePower(void) {
    USB_POWER_CTL_EXT_pin = USB_POWER_CTL_EXT_OFF;
    if(USBdemoState != DISPLAY_REMOVE_MESSAGE )
    {
        asm("reset");  // just a temp fix, need better way to handle removal of thumb drive while writing
    }     
    else
    {
        USBdemoState = DEVICE_DETACHED;
    }

}

/*********************************************************************
 * Function: void APP_HostMSDDataLoggerTasks(void);
 *
 * Overview: This routine prompts the user to insert an USB flash drive
 *          and then writes an incremented value every second.  the user 
 *          can press ok to end the data logging, otherwise data will 
 *          continue to log until the counter reaches 1000.  This
 *          function also catched and displays error messages.
 * PreCondition: The demo should have been initialized via
 *   the APP_HostMSDDataLoggerInitialize(), apply usb power 
            FILEIO_Initialize();
            USBHostInit(0);     
 *
 * Input: None
 *
 * Output: None
 *
 ********************************************************************/
void APP_HostMSDDataLoggerTasks_demo() {
    static uint16_t testData = 0;
    static uint32_t timetick_Ms;
    static int errorCode;
    int charCount;
    if (FILEIO_MediaDetect(&gUSBDrive, &deviceAddress) == false) {
        // The device has been removed.  Now we should wait for a new
        // device to be attached.
        if (allowStateChange)
            USBdemoState = WAITING_FOR_ATTACH;
    }

    switch (USBdemoState) {
        case WAITING_FOR_ATTACH:
            // interrupt routine will call function USB_ApplicationEventHandler 
            // to advance state machine to the MOUNTING_DRIVE state
            break;

        case NON_BLOCK_DELAY_MOUNT_DRIVE:
            if (tick > timetick_Ms + 100) {       // allow some time to fully 
                USBdemoState = MOUNTING_DRIVE;     // insert thumb drive
                createLogFile();                // derived from date
            }                                   // ex: MAR20EDS.CSV
            break;
        case MOUNTING_DRIVE:
        {
            // Attempt to mount the drive described by gUSBDrive as drive 'A'
            // The deviceAddress parameter describes the USB address 
            // of the device; it is initialized by the application in the 
            // USB_ApplicationEventHandler function when a new device is 
            // detected.
            errorCode = FILEIO_DriveMount('A', &gUSBDrive, &deviceAddress);
            if ( errorCode == FILEIO_ERROR_NONE) {
                USBdemoState = OPENING_FILE;
                showScreenTestUSBCapture();
                update_display = GLOBAL_UPDATE;
            } else if (errorCode != FILEIO_ERROR_NOT_PRESENT) {
                USBdemoState = HANDLE_ERROR;
                sprintf((char *) &(pageInformation.instructions2),
                        (char*) "MOUNT: %2d ", errorCode);
            }
            break;
        }
        case OPENING_FILE:
            errorCode = FILEIO_Open(&myFile, Capturename, FILEIO_OPEN_WRITE |
                    FILEIO_OPEN_CREATE |
                    FILEIO_OPEN_APPEND);
            if ( errorCode == FILEIO_RESULT_SUCCESS) {
                timetick_Ms = tick;
                USBdemoState = WRITE_HEADER;
                break;
            } else {
                //  Opening the file failed.  Since we can't write to the
                //  device, abort the write attempt and wait for the device
                //  to be removed 
                USBdemoState = HANDLE_ERROR;
                sprintf((char *) &(pageInformation.instructions2),
                        (char*) "OPEN: %2d ", errorCode);
            }
            break;
        case WRITE_HEADER:
            charCount = fileHeader(1, 2);
            Nop();
            FILEIO_Write(printBuffer, 1, charCount, &myFile);
            USBdemoState = WRITING_TO_FILE;
            break;
        case WRITING_TO_FILE:
            if (tick > (timetick_Ms + 10)) {
                timetick_Ms = tick;
                testData++;
                charCount = sprintf(printBuffer, "%d,%d\r\n", testData, testData + 2);
                FILEIO_Write(printBuffer, 1, charCount, &myFile);
            }

            if (testData > 1000) {
                USBdemoState = CLOSING_FILE;
                testData = 0;
            }
            break;
        case CLOSING_FILE:
            // Always make sure to close the file so that the data gets
            // written to the drive.
            FILEIO_Close(&myFile);
            USBdemoState = UNMOUNT_DRIVE;
            break;
        case UNMOUNT_DRIVE:
            // Unmount the drive since it is no longer in use.
            FILEIO_DriveUnmount('A');
            // Now that we are done writing, we can do nothing until the
            // drive is removed.
            USBdemoState = WAITING_FOR_DETACH;
            break;
        case WAITING_FOR_DETACH:
            sprintf((char *) &(pageInformation.instructions2),
                    (char*) "Remove drive ");
            pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_2);
            GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject, pageInformation.instructions2);
            GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
            update_display = GLOBAL_UPDATE;
            USBdemoState = DISPLAY_REMOVE_MESSAGE;
            allowStateChange = false;
            // an interrupt from _USB1Interrupt() causes the state to advance
            // to DEVICE_DETACTED
            break;
        case DISPLAY_REMOVE_MESSAGE:
            Nop();
            break;
        case DEVICE_DETACHED:
            USBdemoState = WAITING_FOR_ATTACH;
            screenState = SCREEN_DISPLAY_MAIN;
            break;

        case HANDLE_ERROR:
            pObject = GFX_GOL_ObjectFind(ID_INSTRUCTIONS_2);
            GFX_GOL_StaticTextSet((GFX_GOL_STATICTEXT *) pObject, pageInformation.instructions2);
            GFX_GOL_ObjectStateSet(pObject, GFX_GOL_STATICTEXT_DRAW_STATE);
            update_display = GLOBAL_UPDATE;
            timetick_Ms = tick;
            USBdemoState = NON_BLOCK_DELAY;
            break;
        case NON_BLOCK_DELAY:
            if (tick > (timetick_Ms + 10000)) {   // allow time to read error message
                USBdemoState = WAITING_FOR_DETACH;
            }
            break;

        default:
            break;
    }

    USBHostTasks();     // handle device attach/detach and enumeration.
    USBHostMSDTasks();  // maintenance tasks required by the mass storage
    // class
}

void ShowScreenUSBMedia_INIT(void) {
    strcpy(pageInformation.title, "USB CAPTURE");
    InitializeScreen();


    GFX_GOL_StaticTextCreate(ID_MEDIA_REQUEST, // ID
            0, 20, 127, 20 + GFX_TextStringHeightGet((GFX_RESOURCE_HDR *) & Verdana_Bold_10),
            GFX_GOL_STATICTEXT_DRAW_STATE,
            (char *) mediaRequest,
            GFX_ALIGN_CENTER,
            &VB10_Scheme);

}

/*********************************************************************
 * Function: void APP_HostMSDDataLoggerTasks(void);
 *
 * Overview: Keeps the demo running.
 *
 * PreCondition: The demo should have been initialized via
 *   the APP_HostMSDDataLoggerInitialize()  
 *
 * Input: None
 *
 * Output: None
 *
 ********************************************************************/
void APP_HostMSDDataLoggerTasks() {
    static uint16_t data = 0;
    static uint32_t timetick_Ms;
    int charCount;
    if (FILEIO_MediaDetect(&gUSBDrive, &deviceAddress) == false) {
        // The device has been removed.  Now we should wait for a new
        // device to be attached.
        if (allowStateChange)
            USBdemoState = WAITING_FOR_ATTACH;
    }

    switch (USBdemoState) {
        case WAITING_FOR_ATTACH:
            USB_POWER_CTL_EXT_pin = USB_POWER_CTL_EXT_ON;  // turn power back on
            status_1_pin = 0;
            timetick_Ms = tick;
            break;
        case NON_BLOCK_DELAY_MOUNT_DRIVE:
            if (tick > timetick_Ms + 100)        // allow some time to fully 
                USBdemoState = MOUNTING_DRIVE;     // insert thumb drive
            break;
        case MOUNTING_DRIVE:
        {
            // Attempt to mount the drive described by gUSBDrive as drive 'A'
            // The deviceAddress parameter describes the USB address of the 
            // device; it is initialized by the application in the 
            // USB_ApplicationEventHandler function when a new device is detected.
            if (FILEIO_DriveMount('A', &gUSBDrive, &deviceAddress) == FILEIO_ERROR_NONE) {
                USBdemoState = OPENING_FILE;
            }
            break;
        }

        case OPENING_FILE:
            if (FILEIO_Open(&myFile, "LOG_EDS.CSV", FILEIO_OPEN_WRITE |
                    FILEIO_OPEN_CREATE |
                    FILEIO_OPEN_APPEND)
                    == FILEIO_RESULT_SUCCESS) {

                timetick_Ms = tick;
                USBdemoState = WRITE_HEADER;
                break;
            } else {
                //  Opening the file failed.  Since we can't write to the
                //  device, abort the write attempt and wait for the device
                //  to be removed 
            }
            break;
        case WRITE_HEADER:
            charCount = fileHeader(1, 2);
            Nop();
            FILEIO_Write(printBuffer, 1, charCount, &myFile);
            USBdemoState = WRITING_TO_FILE;
            break;

        case WRITING_TO_FILE:

            if (tick > (timetick_Ms + 10)) {
                timetick_Ms = tick;
                data++;
                charCount = sprintf(printBuffer, "%d,%d\r\n", data, data + 2);
                FILEIO_Write(printBuffer, 1, charCount, &myFile);
            }

            if (data > 10) {
                USBdemoState = CLOSING_FILE;
                data = 0;
            }
            break;

        case CLOSING_FILE:
            // Always make sure to close the file so that the data gets
            // written to the drive.
            FILEIO_Close(&myFile);
            USBdemoState = UNMOUNT_DRIVE;
            break;

        case UNMOUNT_DRIVE:
            // Unmount the drive since it is no longer in use.
            FILEIO_DriveUnmount('A');

            // Now that we are done writing, we can do nothing until the
            // drive is removed.
            status_1_pin = 1;
            USBdemoState = WAITING_FOR_DETACH;
            break;
        case WAITING_FOR_DETACH:
            break;
        case DEVICE_DETACHED:
            USBdemoState = WAITING_FOR_ATTACH;
            break;
        default:
            break;
    }
}

int fileHeader(char eventCnt, char event) {

    int charCount = 0;

    charCount += sprintf(printBuffer, "TEST#");

    charCount += addCommaSeparator();
    strncat(printBuffer, "1", 1);
    charCount += 1;
    charCount += addCommaSeparator();
    strncat(printBuffer, "type:" , 5);
    charCount += 5;
    charCount += addCommaSeparator();
    strncat(printBuffer, "1", 1);
    charCount += 2;

    charCount += addTimeStamp();

    return (charCount);
}

int addCommaSeparator() {
    strncat(printBuffer, ",", 1);
    return (1);
}

int addTimeStamp(void) {

    int characterCount = 0;

    RTCCProcessEvents();   // update global variables with new time

    characterCount += addLineTermination();

    strncat(printBuffer, _time_str, 16);
    characterCount += 16;

    characterCount += addCommaSeparator();

    strncat(printBuffer, _date_str, 16);
    characterCount += 16;

    characterCount += addLineTermination();

    return characterCount;
}

int addLineTermination(void) {
    strncat(printBuffer, "\r\n", 2);
    return 2;
}

void createLogFile(void) {

    RTCCProcessEvents();     // refresh date variables

    Capturename[0] = toupper((char) _date_str[0]);
    Capturename[1] = toupper((char) _date_str[1]);
    Capturename[2] = toupper((char) _date_str[2]);


    if (_date_str[4] == ' ')
        Capturename[3] = '0';
    else
        Capturename[3] = _date_str[4];

    Capturename[4] = _date_str[5];
    Capturename[5] =  'E';
    Capturename[6] =  'D';
    Capturename[7] =  'S';
    Capturename[8] =  '.';
    Capturename[9] =  'C';
    Capturename[10] = 'S';
    Capturename[11] = 'V';
    Capturename[12] = 0;
}



///****************************************************************************
//  Function:
//    bool ProcessMessageUSB_test( WORD translatedMsg, GFX_GOL_OBJ_HEADER* pObj,
//                        GFX_GOL_MESSAGE* pMsg )
//
//  Description:
//    This function processes the messages for the usb test selection screen. This
// .
//
//  Precondition:
//    Call showScreenTestUSBCapture() prior to using this function to display the
//    correct screen.
//
//  Parameters:
//    WORD translatedMsg  - The translated control-level message
//    GFX_GOL_OBJ_HEADER* pObj    - Object to which the message applies
//    GFX_GOL_MESSAGE* pMsg       - The original system message information
//
//  Return Values:
//    0   - Do not call the default message handler.
//    1   - Call the default message handler.
//
//  Remarks:
//    global change variable to used  clean up later.
// ***************************************************************************/

bool ProcessMessageUSB_test(WORD translatedMsg, GFX_GOL_OBJ_HEADER* pObj, GFX_GOL_MESSAGE* pMsg) {

    WORD controlID;
    controlID = GFX_GOL_ObjectIDGet(pObj);
    switch (controlID) {
        case ID_BUTTON_GENERIC_HOME:
            if (translatedMsg == GFX_GOL_BUTTON_ACTION_PRESSED) {
                if (USBdemoState == WRITING_TO_FILE) {
                    USBdemoState = CLOSING_FILE;
                    return false; // Hidden button
                }
            } else {
                return false;
            }
            break;
        default:
            return false;
    }

    return true; // Call the default handler to show the button state.
}

