/**
*
* \brief The EPD driver for Microchip Graphic Library
*
* Copyright (c) 2013-2015 Pervasive Displays Inc. All rights reserved.
*
*  Authors: Pervasive Displays Inc.
*
*  Redistribution and use in source and binary forms, with or without
*  modification, are permitted provided that the following conditions
*  are met:
*
*  1. Redistributions of source code must retain the above copyright
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in
*     the documentation and/or other materials provided with the
*     distribution.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <string.h>
#include "system_config.h"
#include <libpic30.h>
#include "drv_gfx_pdi_e_paper.h"
#include "driver/gfx/drv_gfx_display.h"
#include "gfx/gfx_primitive.h"
#include "../src/Pervasive_Displays_small_EPD/conf_EPD.h"

#define DRV_EPD_USE_BUFFERED_PIXEL_PUT    
//#define Use_InternalRAM
#ifdef Use_InternalRAM
#define ImageBufSize    (DISP_HOR_RESOLUTION*(DISP_VER_RESOLUTION/8))
static uint8_t ImageBuf[ImageBufSize];
static uint8_t BK_ImageBuf[ImageBufSize];
#endif
#ifdef GFX_USE_DISPLAY_CONTROLLER_EPD

#ifdef DRV_EPD_USE_BUFFERED_PIXEL_PUT
GFX_STATUS GFX_DRV_DisplayUpdate(uint8_t pixelBit,  uint32_t address, bool sync);
#endif
uint16_t cur_image_index=0;
uint32_t previous_image_address, new_image_address;

static uint8_t bkdata=0xff;

#define	_BV(bit)   (1 << (bit)) /**< left shift 1 bit */

#ifndef BLACK
        #define BLACK	0
#endif
#ifndef WHITE
        #define WHITE	1
#endif

// *****************************************************************************
/*  Function:
    void DRV_GFX_Initialize(void)

    Summary:
        Initialize the graphics display driver.

    Description:
        This function initializes the graphics display driver. This
        function will be called by the application.

*/
// *****************************************************************************
void DRV_GFX_Initialize(void)
{
#ifndef Use_InternalRAM   
    uint32_t i;
    uint8_t ImgData[32];

    //Sets 32 bytes of the block of memory pointed by ImgData to 0xFF
    for(i = 0; i < 32; i++)
    {
        ImgData[i]=0xFF;
    }
    
    //Write 0xFF to SRAM
    for(i = 0; i < 1024; i++)
    {
        DRV_EPD_SRAMWritePage((i*32), ImgData);
    }
    cur_image_index=0;
    new_image_address=getAddress(cur_image_index);
    previous_image_address=getAddress((cur_image_index+1));
#endif
    GFX_ScreenClear();
}

// *****************************************************************************
/*  Function:
    GFX_STATUS GFX_PixelPut(
                    uint16_t    x,
                    uint16_t    y)

    Summary:
        Draw the pixel on the given position.

    Description:
        This routine draws the pixel on the given position.
        The color used is the color set by the last call to
        GFX_ColorSet().

        If position is not on the frame buffer, then the behavior
        is undefined. If color is not set, before this function
        is called, the output is undefined.

*/
// *****************************************************************************



GFX_STATUS GFX_PixelPut(uint16_t x, uint16_t y)
{

    uint32_t    address;
    uint8_t     PixelBit;
#ifndef DRV_EPD_USE_BUFFERED_PIXEL_PUT
    uint8_t     sdata;
#endif

#if (DISP_ORIENTATION == 0)
    PixelBit  = (7 - (x & 0x7));
    address   = (uint32_t)(y*(DISP_HOR_RESOLUTION >> 3)) + (x >> 3);
#else
    y = (GFX_MaxYGet() - y);
    PixelBit = (7 - (y & 0x7));
    address  = (uint32_t)(x*(DISP_HOR_RESOLUTION >> 3)) + (y >> 3);
#endif
    
#ifdef Use_InternalRAM
    if(GFX_ColorGet() == BLACK)  {
        ImageBuf[address] &=(~_BV(PixelBit));
    }else
    {
        ImageBuf[address] |= _BV(PixelBit);
    }
#else
    //address page offset
    address  += new_image_address;
#ifdef DRV_EPD_USE_BUFFERED_PIXEL_PUT
    GFX_DRV_DisplayUpdate(PixelBit, address, false);
#else
    bkdata    = DRV_EPD_SRAMReadByte(address);

    if(GFX_ColorGet() == BLACK)  {
        sdata = (bkdata & (~_BV(PixelBit)));
    }else
    {
        sdata=(bkdata | _BV(PixelBit));
    }

    if(sdata != bkdata)
    {
        DRV_EPD_SRAMWriteByte(address, sdata);
        bkdata = sdata;
    }
#endif
#endif
    return (GFX_STATUS_SUCCESS);
}
    
#ifdef DRV_EPD_USE_BUFFERED_PIXEL_PUT
GFX_STATUS GFX_DRV_DisplayUpdate(uint8_t pixelBit,  uint32_t address, bool sync)
{
    static uint32_t lastAddress = 0xFFFFFFFF;
    static uint8_t  lastByte = 0;
    uint8_t currentByte;

    // check if we are doing synchronization only
    // this means the contents of the lastByte must be updated to the screen
    // when sync is true, both pixelBit and address parameters are ignored
    if (sync == true)
    {
        DRV_EPD_SRAMWriteByte(lastAddress, lastByte);
        lastAddress = 0xFFFFFFFF;
        lastByte = 0;
        return(GFX_STATUS_SUCCESS);  // todo what is the return value???
        // return value not used. PDI did not specify a return value
        // I put a return value to satisfy the compiler
    }

    // =====================================================
    // on this portion the pixels are only updated when the
    // address is not matching the previous address
    // =====================================================

    // check if address is the same as before call to this function
    // if the same, update only the the current buffer
    if (lastAddress == address)
    {
        // no need to fetch, pixel to be updated is
        // on the lastByte already
        currentByte = lastByte;
    }
    else
    {
        // write the last byte to update then load the new byte
        // from the given new address

        if (lastAddress != 0xFFFFFFFF)
        {
            // last address is valid, upload the lastByte first
            DRV_EPD_SRAMWriteByte(lastAddress, lastByte);
        }
        // get the new address
        lastAddress = address;
        // now load the new byte
        currentByte = DRV_EPD_SRAMReadByte(address);
    }
    if(GFX_ColorGet() == BLACK)  {
        currentByte = (currentByte & (~_BV(pixelBit)));
    }
    else
    {
        currentByte = (currentByte | _BV(pixelBit));
    }
    // update the buffer
    lastByte = currentByte;
    return (GFX_STATUS_SUCCESS);
}
#endif
// *****************************************************************************
/*  Function:
    GFX_COLOR GFX_PixelGet(
                    uint16_t    x,
                    uint16_t    y)

    Summary:
        Gets color of the pixel on the given position.

    Description:
        This routine gets the pixel on the given position.

        If position is not on the frame buffer, then the behavior
        is undefined.

*/
// *****************************************************************************
GFX_COLOR GFX_PixelGet(uint16_t x, uint16_t y)
{
    // NOTE: This is currently not supported
    //       and will always return 1.

    return 1;

}

// *****************************************************************************
/*
    <GROUP driver_rendering_functions>

    Function:
        GFX_STATUS_BITS GFX_RenderStatusGet()

    Summary:
        This function returns the driver's status on rendering.

    Description:
        This function returns the driver's rendering status. Valid values
        returned are GFX_STATUS_BUSY_BIT or GFX_STATUS_READY_BIT
        (see GFX_STATUS_BITS).

*/
// *****************************************************************************
GFX_STATUS_BIT GFX_RenderStatusGet(void)
{
    return GFX_STATUS_READY_BIT;
}


// *****************************************************************************
/*  Function:
    GFX_STATUS GFX_ScreenClear(void)

    Summary:
        Clears the screen to the currently set color (GFX_ColorSet()).

    Description:
        This function clears the screen with the current color and sets
        the line cursor position to (0, 0).

        If color is not set, before this function is called, the output
        is undefined.

        If the function returns GFX_STATUS_FAILURE, clearing is not
        yet finished. Application must call the function again to continue
        the clearing.

*/
// *****************************************************************************
GFX_STATUS GFX_ScreenClear(void)
{
    uint32_t    i;
    uint8_t     tmp = 0xFF;
    

    // SpiRAM_Init();
    if(GFX_ColorGet() == BLACK)
        tmp = 0x00;
#ifdef Use_InternalRAM
     for(i = 0; i< ImageBufSize; i++)
    {
         ImageBuf[i]=tmp;
    }
#else
    uint8_t     ImgData[32];
    for(i = 0; i < 32; i++)
    {
        ImgData[i] = tmp;
    }
    for(i = 0; i< _epd_page_size(); i++)
    {
        DRV_EPD_SRAMWritePage((i*32), ImgData);
    }
#endif
    GFX_LinePositionSet(0, 0);

    return GFX_STATUS_SUCCESS;  
}

/**
 * Store the last updated image to Previous image
 */
void DRV_EPD_StoreScreen(void){

    uint32_t i;
#ifdef Use_InternalRAM
    for(i = 0; i< ImageBufSize; i++)
    {
        BK_ImageBuf[i]= ImageBuf[i];
    }
#else
    uint8_t ImgData[32];
    // SpiRAM_Init();
    for(i = 0; i < _epd_page_size(); i++)
    {
        DRV_EPD_SRAMReadPage((i*32), ImgData);
        DRV_EPD_SRAMWritePage((i*32)+previous_image_address,ImgData);
    }
#endif
}

/**
 * Read image data from SRAM
 * @param memory_address start address of memory to read
 * @param target_buffer the buffer of read data
 * @param byte_length the total length to read
 */
void DRV_EPD_SRAMHandleRead(    EInt memory_address,
                                uint8_t *target_buffer,
                                uint8_t byte_length)
{
    DRV_EPD_SRAMReadSequential( memory_address,
                                target_buffer,
                                byte_length);
}

/**
 * EPD global update function
 */
void DRV_EPD_UpdateGlobal(void)
{
#ifdef Use_InternalRAM
    EPD_display_global_RAM(USE_EPD_Type,(uint8_t *)&BK_ImageBuf,(uint8_t *)&ImageBuf);
#else
#ifdef DRV_EPD_USE_BUFFERED_PIXEL_PUT
    GFX_DRV_DisplayUpdate(0, new_image_address, true);
#endif
    EPD_display_global( USE_EPD_Type,
                        previous_image_address,
                        new_image_address,
                        DRV_EPD_SRAMHandleRead);
#endif
    DRV_EPD_StoreScreen();
}

/**
 * EPD power on and initialization
 */
void DRV_EPD_PowerUp(void)
{
    if (EPD_PowerStatusGet() != true)
        EPD_power_init(USE_EPD_Type);
}

/**
 * EPD power down
 */
void DRV_EPD_PowerDown(void)
{
    if (EPD_PowerStatusGet() != false)
        EPD_power_end();
}

/**
 * EPD partial update function
 */
void DRV_EPD_UpdatePartial(void)
{
#ifdef Use_InternalRAM
    EPD_display_partial_RAM(USE_EPD_Type,(uint8_t *)&BK_ImageBuf,(uint8_t *)&ImageBuf);
#else
#ifdef DRV_EPD_USE_BUFFERED_PIXEL_PUT
    GFX_DRV_DisplayUpdate(0, new_image_address, true);
#endif
    EPD_display_partial(USE_EPD_Type,
                        previous_image_address,
                        new_image_address,
                        DRV_EPD_SRAMHandleRead);
#endif
    DRV_EPD_StoreScreen();
}

void DRV_GFX_UpdateBiStableScreen(bool fullScreen,bool IspowerDown)
{
    if (fullScreen == true)
        DRV_EPD_UpdateGlobal();
    else
    {
#ifdef G2_Aurora_Ma
        DRV_EPD_UpdateGlobal();
#else
        DRV_EPD_PowerUp();
        DRV_EPD_UpdatePartial();
        if(IspowerDown)
        DRV_EPD_PowerDown();
#endif
    }

}


#endif

